// ---------------------------------------------------------------------------------------------------------------------------- \\
//															        \\
// File: GTEST.FTH      Author: Oliver SINGLA	 Date: Nov,1,94	    Version: 2.00					        \\
// ----		        ------			 ----		    -------						        \\
//															        \\
// Description:														        \\
//     Graphics test.													        \\
//															        \\
// Release:														        \\
//     2.00    Oct-02-95												        \\
//															        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\

( Force decimal for integer numeric I/O )
decimal

( Compiler strategy )
?speed true!						       // Choose speed vs. size
?safe true!						       // Run-time check stack under/overflow
?iarithoverflow true!					       // Run-time check integer arithmetic overflow
?chat true!						       // Print a dot for each compiled definition

( Empty word: this is the 1st word, we kill on each load )
forget <task>
: <task> ;

( Colors used )
screen
black LightGray BackForeGround const C_NormalText	       // Normal text

( Some constants )
50000 const NB_PIXELS					       // Nb of random pixels
int gnHeightTittle					       // Height of tittle zone


( Make GRAPHICS the context vocabulary )
graphics


: point>
  build>						       // Compile-time: ( n --- )
      2* 2* ballot drop					       // Room for <n> couples of word
  does>							       //     Run-time: ( n --- addr )	 <addr> is address of x-point
      swap 2* 2* +					       // ( n addr1 --- addr2 )
  ;  // point>


: ComputeHeightTittle
  #grows screen #rows graphics /
  gnHeightTittle !
  ;  // ComputeHeightTittle


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PressKey												        \\
// 	 Stack : ( --- )												        \\
// Description : Type a msg, then wait until a key is pressed.								        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: PressKey
  screen
  #cols 11 - 0 " Press a key" !xystring
  key drop
  #cols 11 - 0 gotoxy clreol
  graphics
  ;  // PressKey


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawSomePixels												        \\
// 	 Stack : ( addrz --- )												        \\
// Description : Draw some pixels.											        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: DrawSomePixels

  ComputeHeightTittle
  screen
  0 0 gotoxy
  !string
  " : SetPixel / GetPixel" !string
  graphics

  NB_PIXELS point> points				       // All points pixel-ed

  ( Draw some pixels, remember each pixel )
  0  NB_PIXELS 1-  make
      0	 #gcols 1-  rnd					       // ( --- x )
      gnHeightTittle @  #grows 1-  rnd			       // ( x --- x y )
      ddup						       // ( x y --- x y x y )
      0	 #gcolors 1-  rnd				       // ( x y x y --- x y x y color )
      SetPixel						       // ( x y x y color --- x y )
      I points						       // ( x y --- x y addr )
      rot over W!					       // ( x y addr --- y addr )
      2+ W!						       // ( y addr --- )
  endmake

  ( Restore each pixel with its previous color )
  0  NB_PIXELS 1-  make
      I points						       // ( --- addr )
      dup W@ swap					       // ( addr --- x addr )
      2+ W@						       // ( x addr --- x y )
      0
      SetPixel						       // ( x y color --- )
  endmake

  PressKey						       // Pause

  ;  // DrawSomePixels


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PerformTest_pixels											        \\
// 	 Stack : ( c --- )  <c> is the graphics mode.									        \\
// Description : Test pixels definitions.										        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: PerformTest_pixels
    case
        'A of  " 320x200x16"  DrawSomePixels  endof
        'B of  " 640x200x16"  DrawSomePixels  endof
        'C of  " 640x350x4"   DrawSomePixels  endof
        'D of  " 640x350x16"  DrawSomePixels  endof
        'E of  " 640x480x2"   DrawSomePixels  endof
        'F of  " 640x480x16"  DrawSomePixels  endof
        'G of  " 320x200x256" DrawSomePixels  endof
        'H of  " 800x600x16"  DrawSomePixels  endof
    endcase
    ;  // PerformTest_pixels


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawSomeLines												        \\
// 	 Stack : ( addrz --- )												        \\
// Description : Draw some lines.											        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: DrawSomeLines

  ComputeHeightTittle
  screen
  0 0 gotoxy
  !string
  " : lines" !string
  graphics

  int nColor1
  int nColor2
  10 times

      0 #gcolors 1- rnd	 nColor1 !
      gnHeightTittle @  #grows 1-  make
	  0  gnHeightTittle @  #gcols 1-  I  nColor1 @  line
      endmake

      begin
	  0 #gcolors 1- rnd  nColor2 !
	  nColor2 nColor1 @!=
      until
      #gcols 1-	 0  -1  +make
	  0  gnHeightTittle @  I  #grows 1-  nColor2 @  line
      endmake

  endtimes

  PressKey						       // Pause

  ;  // DrawSomeLines


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PerformTest_lines											        \\
// 	 Stack : ( c --- )  <c> is the graphics mode.									        \\
// Description : Test lines definitions.										        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: PerformTest_lines
    case
        'A of  " 320x200x16"  DrawSomeLines  endof
        'B of  " 640x200x16"  DrawSomeLines  endof
        'C of  " 640x350x4"   DrawSomeLines  endof
        'D of  " 640x350x16"  DrawSomeLines  endof
        'E of  " 640x480x2"   DrawSomeLines  endof
        'F of  " 640x480x16"  DrawSomeLines  endof
        'G of  " 320x200x256" DrawSomeLines  endof
        'H of  " 800x600x16"  DrawSomeLines  endof
    endcase
    ;  // PerformTest_lines


: DrawSomeBoxes

  ComputeHeightTittle
  screen
  0 0 gotoxy
  !string
  " : boxes" !string
  graphics

  750 times
      0	 #gcols 1-  rnd
      gnHeightTittle @  #grows 1-  rnd
      0	 #gcols 1-  rnd
      gnHeightTittle @  #grows 1-  rnd
      0	 #gcolors 1-  rnd
      FILLED
      box
  endtimes

  PressKey						       // Pause

  ;  // DrawSomeBoxes


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PerformTest_boxes											        \\
// 	 Stack : ( c --- )  <c> is the graphics mode.									        \\
// Description : Test boxes definitions.										        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:   PerformTest_boxes
    case
        'A of  " 320x200x16"  DrawSomeBoxes  endof
        'B of  " 640x200x16"  DrawSomeBoxes  endof
        'C of  " 640x350x4"   DrawSomeBoxes  endof
        'D of  " 640x350x16"  DrawSomeBoxes  endof
        'E of  " 640x480x2"   DrawSomeBoxes  endof
        'F of  " 640x480x16"  DrawSomeBoxes  endof
        'G of  " 320x200x256" DrawSomeBoxes  endof
        'H of  " 800x600x16"  DrawSomeBoxes  endof
    endcase
    ;  // PerformTest_boxes


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawTittle												        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the tittle on screen										        \\
//    Callers : Frame													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: DrawTittle
  screen
  24 0 " Ŀ" !xystring
  24 1 "  GRAPHICS EXAMPLES AND TESTS " !xystring
  24 2 " " !xystring
  ;  // DrawTittle


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawTestsMenu												        \\
// 	 Stack : ( x y --- )												        \\
// Description : Draw the tests menu.											        \\
//    Callers : Frame													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: DrawTestsMenu
  const y
  const x
  '~ >xylstring_char
  0 black yellow    >xylstring_attr
  1 black lightgray >xylstring_attr
  x y	   " ~{1}Ŀ" !xylstring
  x y  1 + " ~{1}       TESTS       " !xylstring
  x y  2 + " ~{1}Ĵ" !xylstring
  x y  3 + " ~{1}   ~{0}1~{1}. Pixels       " !xylstring
  x y  4 + " ~{1}   ~{0}2~{1}. Lines        " !xylstring
  x y  5 + " ~{1}   ~{0}3~{1}. Boxes        " !xylstring
  x y  6 + " ~{1}   ~{0}4~{1}. Sprites      " !xylstring
  x y  7 + " ~{1}Ĵ" !xylstring
  x y  8 + " ~{1} ~{0}ESC~{1}. Exit         " !xylstring
  x y  9 + " ~{1}" !xylstring
  ;  // DrawTestsMenu


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawResolutionsMenu											        \\
// 	 Stack : ( x y --- )												        \\
// Description : Draw the resolutions menu.										        \\
//    Callers : Frame													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: DrawResolutionsMenu
  const y
  const x
  '~ >xylstring_char
  0 black yellow    >xylstring_attr
  1 black lightgray >xylstring_attr
  x y  0 + " ~{1}Ŀ" !xylstring
  x y  1 + " ~{1}     RESOLUTIONS     " !xylstring
  x y  2 + " ~{1}Ĵ" !xylstring
  x y  3 + " ~{1} ~{0}A~{1}. 320 x 200 x 16   " !xylstring
  x y  4 + " ~{1} ~{0}B~{1}. 640 x 200 x 16   " !xylstring
  x y  5 + " ~{1} ~{0}C~{1}. 640 x 350 x 4    " !xylstring
  x y  6 + " ~{1} ~{0}D~{1}. 640 x 350 x 16   " !xylstring
  x y  7 + " ~{1} ~{0}E~{1}. 640 x 480 x 2    " !xylstring
  x y  8 + " ~{1} ~{0}F~{1}. 640 x 480 x 16   " !xylstring
  x y  9 + " ~{1} ~{0}G~{1}. 320 x 200 x 256  " !xylstring
  x y 10 + " ~{1} ~{0}H~{1}. 800 x 600 x 16   " !xylstring
  x y 11 + " ~{1}" !xylstring
  ;  // DrawResolutionsMenu


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : FixResolution												        \\
// 	 Stack : ( c --- )												        \\
// Description : Fix the resolution, as given in DrawResolutionsMenu							        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: FixResolution
    graphics
    case
        'A of  320x200x16  endof
        'B of  640x200x16  endof
        'C of  640x350x4   endof
        'D of  640x350x16  endof
        'E of  640x480x2   endof
        'F of  640x480x16  endof
        'G of  320x200x256 endof
        'H of  0x58 800x600x16  endof
    endcase
    screen
    ;  // FixResolution


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PerformTest												        \\
// 	 Stack : ( c1 c2 --- )	 <c1> is graphics mode	  <c2> is operation						        \\
// Description : Perform the asked test (as given in DrawTestsMenu).							        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: PerformTest
    case
        '1 of  PerformTest_Pixels      endof
        '2 of  PerformTest_Lines       endof
        '3 of  PerformTest_boxes       endof
    endcase
    ; // PerformTest


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : InputAnswer												        \\
// 	 Stack : ( x y --- tf )	  if exit application (ESC)								        \\
// 		 ( x y --- n ff ) if an operation is asked.								        \\
// Description : Draw input box, then wait for answer.									        \\
//    Callers : Frame													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: InputAnswer

  const y
  const x
  '~ >xylstring_char
  0 black yellow    >xylstring_attr
  1 black lightgray >xylstring_attr
  x y	  " ~{1}Ŀ" !xylstring
  x y 1 + " ~{1} Enter your choice (example: ~{0}A1~{1} or ~{0}1A~{1}) ?    " !xylstring
  x y 2 + " ~{1}" !xylstring

  x 42 + const x
  y 1+	 const y
  FALSE ilogical lExit?					       // Exit loop ?
  FALSE ilogical lEsc?					       // Exit application ?
  0 iint nIndex						       // Index of key to input
  begin

      nIndex @ case

	  0 of
	      x y ddup 0x20 !xychar  gotoxy  key charuppercase  const cKey0
	      cKey0 0x1B == if
		  lEsc? true!
	      else
		  cKey0 '1 '4 >=<=  cKey0 'A 'H >=<=  or  if
		      cKey0 emit
		      nIndex 1+!
		  endif
	      endif
	  endof

	  1 of
	      x 1+ y ddup 0x20 !xychar  gotoxy  key charuppercase  const cKey1
	      cKey1 0x1B == if
		  lEsc? true!
	      else
		  cKey1 0x08 == if
		      x y 0x20 !xychar  nIndex 1-!
		  else
		      ckey1  cKey0 '1 '4 >=<= if
			  'A 'H
		      else
			 '1 '4
		      endif
		      >=<=  if
			  cKey1 emit
			  nIndex 1+!
		      endif
		  endif
	      endif
	  endof

	  2 of
	      x 2 + y ddup 0x20 !xychar	 gotoxy	 key
	      case
		  0x1B of  lEsc? true!			    endof
		  0x0D of  lExit? true!			    endof
		  0x08 of  x 1+ y 0x20 !xychar  nIndex 1-!  endof
	      endcase
	  endof

      endcase

      lExit? true?  lEsc? true?	 or
  until

  lEsc? true? if
      TRUE
  else
      cKey0 '1 '4 >=<= if
	  cKey1 FixResolution
	  cKey1 cKey0 PerformTest
      else
	  cKey0 FixResolution
	  cKey0 cKey1 PerformTest
      endif
      FALSE
  endif

  ;  // InputAnswer


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Menu													        \\
// 	 Stack : ( --- f )    <f> : exit application ?									        \\
// Description : Draw menus, wait then perform asked operation, then exit.						        \\
//    Callers : Monitor												        \\
// 	 Calls : DrawTittle DrawActionsFrame DrawInfosFrame								        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: Menu
  3 SetVideoMode					       // 25x80x16, text mode, VGA
  DrawTittle						       // Draw tittle application.
  12 4 DrawResolutionsMenu				       // Draw the menu of available resolutions
  42 4 DrawTestsMenu					       // Draw the menu of available tests
  17 #rows 3 - InputAnswer				       // ( --- n )
  if
      TRUE						       // Exit application
  else
      FALSE						       // Do not exit application
  endif
  ;  // Menu


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : gtest													        \\
// 	 Stack : ( --- )												        \\
// Description : Start the graphics definitions test.									        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: gtest
  GetVideoMode						       // ( --- n )
  3 SetVideoMode					       // 25x80x16, text mode, VGA
  C_NormalText ScreenBackground !			       // The screenbackground is Black
  cls							       // Clear screen
  ?UpdateCursor true!					       // Update cursor when printing on screen
  begin
      menu						       // Perform asked operation
  until
drop //	 SetVideoMode						      // ( n --- )
  ;  // gtest

: g gtest 0 15 gotoxy ;


( Print informations about Code/Data size )
( --------------------------------------- )

infos

