// ---------------------------------------------------------------------------------------------------------------------------- \\
// Description : Adds the new word <addrz1> to the string <addrz2>.							        \\
//															        \\
// File: HANGMAN.FTH    Author: Oliver SINGLA	 Date: Oct,19,95    Version: 1.00					        \\
// ----		        ------			 ----		    -------						        \\
//															        \\
// Description:	 Hangman game.												        \\
//															        \\
// Release:														        \\
//															        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\

( Force decimal for integer numeric I/O )
decimal

( Compiler strategy )
?speed true!						       // Choose speed vs. size
?safe true!						       // Run-time check stack under/overflow
?iarithoverflow true!					       // Run-time check integer arithmetic overflow
?chat true!						       // Print a dot for each compiled definition

( Empty word: this is the 1st word, we kill on each load )
forget <task>
:  <task> ;

( Make SCREEN the context vocabulary )
screen

( Colors used )
black LightGray BackForeGround const C_NormalText	       // Normal text

( Position of elements )
int x_Gallows
int y_Gallows

( The list of words )
127 32 dstring FrenchWords$    " [000]" FrenchWords$  $!
127 32 dstring EnglishWords$   " [000]" EnglishWords$ $!

( The word to guess )
48 fstring WordToGuess$

( The word guessed )
48 fstring WordGuessed$

( The level of death )
int gnDeathLevel
8 const gnMaxDeathLevel

// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : --word->												        \\
// 	 Stack : ( addrz1 addrz2 --- )											        \\
// Description : Adds the new word <addrz1> to the string <addrz2>.							        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: --word->
   dup 1 $ConvNb under  const n
   n <# '] hold # # # '[ hold #> swap $+
   $+

   ( Update now the number of words into the list )
   n 1+ <# # # # #> swap 1 $replace

   ;   // --word->

( List of french words )
" Television"       FrenchWords$  --word->
" Journal"          FrenchWords$  --word->
" Cendrier"         FrenchWords$  --word->
" Dossier"          FrenchWords$  --word->
" Chaise"           FrenchWords$  --word->
" Placart"          FrenchWords$  --word->
" Bruissement"      FrenchWords$  --word->
" Souris"           FrenchWords$  --word->
" Ruisseau"         FrenchWords$  --word->
" Soupiere"         FrenchWords$  --word->
" Agissement"       FrenchWords$  --word->
" Radiateur"        FrenchWords$  --word->
" Chauffage"        FrenchWords$  --word->
" Hurlement"        FrenchWords$  --word->
" Documentaliste"   FrenchWords$  --word->
" Certainement"     FrenchWords$  --word->
" Mutation"         FrenchWords$  --word->
" Garnement"        FrenchWords$  --word->
" Myopie"           FrenchWords$  --word->
" Narquois"         FrenchWords$  --word->
" Nauseabond"       FrenchWords$  --word->
" Allocation"       FrenchWords$  --word->

( List of english words )
" Television"          EnglishWords$ --word->
" Salad"               EnglishWords$ --word->
" Computer"            EnglishWords$ --word->
" Jack"                EnglishWords$ --word->
" Eyelid"              EnglishWords$ --word->
" Tractor"             EnglishWords$ --word->
" Thingumajig"         EnglishWords$ --word->
" Wolf"                EnglishWords$ --word->
" Fork"                EnglishWords$ --word->
" Spoon"               EnglishWords$ --word->
" Book"                EnglishWords$ --word->
" Picture"             EnglishWords$ --word->

forget --word->


( Keys used )
FKeys
KEY_ESC const KEY_BYE					       // ESC : Bye application
KEY_F2  const KEY_CHANGE_LANG				       // F2  : Change the language
Screen


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : larray													        \\
// 	 Stack : Compile-time: ( n --- )										        \\
// 		     Run-time: ( n --- addr )										        \\
// Description : To manage a one-dimension array of logical.								        \\
//    Callers : Letters												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  larray
   build>						       // COMPILE-TIME:
       ballot drop
   does>						       // RUN-TIME:
       +						       // ( n addr1 --- addr2 )
   ;   // larray

( Flag for each letter )
26 larray Letters


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Yes?													        \\
// 	 Stack : ( c --- f )												        \\
// Description : Is-is the YES key ?											        \\
//    Callers : AnotherGame?												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Yes?
   language @ case
       french of  'O == endof
		  'Y ==
   endcase
   ;   // Yes?


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : YesNo?													        \\
// 	 Stack : ( c --- f )												        \\
// Description : Is-is the YES or NO keys ?										        \\
//    Callers : AnotherGame?												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  YesNo?
   language @ case
       french of  dup 'O == swap 'N == or endof
		  dup 'Y == swap 'N == or
   endcase
   ;   // YesNo?


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : doTittle												        \\
// 	 Stack : ( addrz --- )												        \\
// Description : Draw the tittle of the game										        \\
//    Callers : Tittle													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  doTittle
   dup $len const len
   #cols  len 4 + -  2/	 const  x
   x 0 ' !xychar  x 1+ 0 len 2+ ' !xynchar  x len + 3 + 0 ' !xychar
   x 1 "   " !xystring  x 2+ 1 rot !xystring  x len + 2+ 1 "  " !xystring
   x 2 ' !xychar  x 1+ 2 len 2+ ' !xynchar  x len + 3 + 2 ' !xychar
   ;   // doTittle


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : tittle													        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the tittle of the game										        \\
//    Callers : Frame													        \\
// 	 Calls : doTittle												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Tittle
   language @ case
       french of  " Jeu du PENDU"  endof
		  " HANGMAN GAME"
   endcase
   doTittle
   ;   // Tittle


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : "WordToGuess"                                                                                                  \\
// 	 Stack : ( --- )												        \\
// Description : Print the text "Word to guess"                                                                                 \\
//    Callers : Frame													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  "WordToGuess"
   language @ case
       french of  "       Mot \85 deviner : "  endof
		  "       Word to guess : "
   endcase
   0 6 rot !xystring					       // Show text
   ;   // "WordToGuess"


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : "LettersProposed"                                                                                              \\
// 	 Stack : ( --- )												        \\
// Description : Print the text "Letters proposed"                                                                              \\
//    Callers : Frame													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: "LettersProposed"
   language @ case
       french of  "   Lettres propos\82es : "  endof
		  "    Letters proposed : "
   endcase
   0 8 rot !xystring					       // Show text
   ;   // "LettersProposed"


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : "YourChoice"                                                                                                   \\
// 	 Stack : ( --- )												        \\
// Description : Print the given text.											        \\
//    Callers : Frame													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  "YourChoice"
   language @ case
       french of  "         Votre choix : "  endof
		  "         Your choice : "
   endcase
   0 10 rot !xystring					        // Show text
   ;   // "YourChoice"


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ShowAvailableKeysCmd											        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the available keys.										        \\
//    Callers : Frame													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ShowAvailableKeysCmd
   '~ >xylstring_char
   0 black white       >xylstring_attr
   1 black lightgray   >xylstring_attr
   0 #rows 1-						       // ( --- x y )
   language @ case
       french   of  "  ~{0}F1~{1} aide   ~{0}F2~{1} anglais   ~{0}ESC~{1} au-revoir"   endof
		    "  ~{0}F1~{1} help   ~{0}F2~{1} french   ~{0}ESC~{1} bye"
   endcase
   !xylstring
   clreol						       // Clear until end of line
   ;  // ShowAvailableKeysCmd


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawGallows												        \\
// 	 Stack : ( n --- )												        \\
// Description : Draw the gallows.											        \\
//    Callers : Frame AnalyseLetter											        \\
// 	 Calls : x_Gallows y_Gallows											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  DrawGallows
   x_Gallows @ const x
   y_Gallows @ const y
   case
       1 of
	   x y 8 '- !xynchar
	   0 6 make  x y 1+ i + 'I !xychar  endmake
	   x y  8 + 12 '- !xynchar
	   x y  9 + 12 'I !xynchar
	   x y 10 + 12 '- !xynchar
       endof
       2 of
	   x 7 +  y 1+  'I  !xychar
	   x 7 +  y 2+  'O  !xychar
       endof
       3 of
	   x 7 +  y 3 +	 'I  !xychar
       endof
       4 of
	   x 6 +  y 3 +	 " /I\\" !xystring
       endof
       5 of
	   x 5 +  y 3 +	 " \\/I\\/" !xystring
       endof
       6 of
	   x 7 +  y 4 +	 'I  !xychar
       endof
       7 of
	   x 6 +  y 5 +	 " /I\\" !xystring
       endof
       8 of
	   x 5 +  y 5 +	 " \\/I\\/" !xystring
       endof
   endcase
   ;   // DrawGallows


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ChooseAWord												        \\
// 	 Stack : ( --- addrz )												        \\
// Description : Choose randomly a new word										        \\
//    Callers : InitGame												        \\
// 	 Calls : FrenchWords$ EnglishWords$ WordToGuess$								        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ChooseAWord
   language @ french ==
   if FrenchWords$ else EnglishWords$ endif
   dup 1 $ConvNb under					       // ( addrz --- addrz n )
   1- 0 swap rnd					       // ( addrz n1 --- addrz n2 )
   <# '] hold # # # '[ hold #> swap $find
   5 + '[ WordToGuess$ $CopyToChar			       // Copy word to guess from the list
   ;   // ChooseAWord


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : PrepareWordToGuess											        \\
// 	 Stack : ( --- )												        \\
// Description : Prepare the word to guess (set with '.').								        \\
//    Callers : InitGame												        \\
// 	 Calls : WordToGuess$:3 WordGuessed$:3										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  PrepareWordToGuess
   WordToGuess$ WordGuessed$ $!
   WordToGuess$ $len 0 do
       '. WordGuessed$ I + C!
   enddo
   0  WordGuessed$ WordToGuess$ $len +  C!
   ;   // PrepareWordToGuess


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : InitGame												        \\
// 	 Stack : ( --- )												        \\
// Description : Start a new game.											        \\
//    Callers : Hangman												        \\
// 	 Calls : gnDeathLevel Letters ChooseAWord PrepareWordToGuess							        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  InitGame
   ChooseAWord						       // Select randomly a word from the list
   PrepareWordToGuess
   26 0 do  I Letters false!  enddo			       // None letter yet proposed
   gnDeathLevel 0!					       // Not yet on the Gallows...
   ;   // InitGame


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawWordToGuess											        \\
// 	 Stack : ( --- )												        \\
// Description : Print the word to guess.										        \\
//    Callers : Frame UpdateWordGuessed Game										        \\
// 	 Calls : WordGuessed$												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  DrawWordToGuess
   22 6 WordGuessed$ !xystring
   ;   // DrawWordToGuess


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ShowLettersProposed											        \\
// 	 Stack : ( --- )												        \\
// Description : Print all the letters proposed, in alphabetical order.							        \\
//    Callers : Frame AnalyseLetter											        \\
// 	 Calls : Letters												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ShowLettersProposed
   26 fstring list$
   0
   'A 'Z make
       I 'A - Letters true? if
	   I  over list$ +  C!
	   1+
       endif
   endmake
   list$ + 0 swap C!
//   22 8 list$ !xystring
   22 8 gotoxy list$ type
   ;   // ShowLettersProposed


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Frame													        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the frame (tittle + rules) of the game.								        \\
//    Callers : AskLetter AnotherGame? Hangman										        \\
// 	 Calls : x_Gallows y_Gallows gnDeathLevel Tittle "WordToGuess" "LettersProposed" "YourChoice" ShowAvailableKeysCmd      \\
// 		 DrawGallows DrawWordToGuess ShowLettersProposed							        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Frame
   cls							       // Clear screen
   Tittle						       // Draw tittle game
   "WordToGuess"                                               // "Word to guess"
   "LettersProposed"                                           // "Letters proposed"
   "YourChoice"
   ShowAvailableKeysCmd
   #cols 12 - 8 - x_Gallows !
   5 y_Gallows !
   ShowLettersProposed
   DrawWordToGuess
   1 gnDeathLevel @ make
       I DrawGallows
   endmake
   ;   // Frame


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : AskLetter												        \\
// 	 Stack : ( --- c )												        \\
// Description : Ask for the choice of the player (one letter).								        \\
//    Callers : Game													        \\
// 	 Calls : KEY_CHANGE_LANG Letters Frame										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  AskLetter
   false ilogical ?done
   begin
       22 10 gotoxy
       key charuppercase const k0
       k0 KEY_CHANGE_LANG == if
	   1 language @ - language !
	   Frame
       else
	   k0 'A 'Z >=<= if
	       22 10 k0 !xychar
	       k0 'A - Letters false? ?done C!
	   endif
       endif
       ?done true?
   until
   k0
   ;   // AskLetter


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : UpdateLettersProposed											        \\
// 	 Stack : ( c --- )												        \\
// Description : Update the flag Proposed? for the proposed letter.							        \\
//    Callers : AnalyseLetter												        \\
// 	 Calls : Letters												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  UpdateLettersProposed
   'A - Letters true!
   ;   // UpdateLettersProposed


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : IsLetterPresent?											        \\
// 	 Stack : ( c --- )												        \\
// Description : Search if the given letter is in the word to guess.							        \\
//    Callers : AnalyseLetter												        \\
// 	 Calls : WordToGuess$:2												        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  IsLetterPresent?
   const k0
   false ilogical ?present
   WordToGuess$ $len 0 do
       WordToGuess$ I + C@ charuppercase k0 == if
	   ?present true!
       endif
   enddo
   ?present true?
   ;   // IsLetterPresent?


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : UpdateWordGuessed											        \\
// 	 Stack : ( c --- )												        \\
// Description : Update the word guessed with the given letter.								        \\
//    Callers : AnalyseLetter												        \\
// 	 Calls : WordToGuess$:3 WordGuessed$ DrawWordToGuess								        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  UpdateWordGuessed
   const letter
   WordToGuess$ $len 0 do
       WordToGuess$ I + C@ CharUppercase  letter  ==  if
	   WordToGuess$ I + C@  WordGuessed$ I +  C!
       endif
   enddo
   DrawWordToGuess
   ;   // UpdateWordGuessed


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : AnalyseLetter												        \\
// 	 Stack : ( c --- )												        \\
// Description : Analyse the choice (letter) given by the player.							        \\
//    Callers : Game													        \\
// 	 Calls : gnDeathLevel:2 DrawGallows ShowLettersProposed UpdateLettersProposed IsLetterPresent? UpdateWordGuessed        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  AnalyseLetter
   const k0
   k0 UpdateLettersProposed
   ShowLettersProposed
   k0 IsLetterPresent? if
       k0 UpdateWordGuessed
   else
       gnDeathLevel 1+!
       gnDeathLevel @ DrawGallows
   endif
   ;   // AnalyseLetter


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Game													        \\
// 	 Stack : ( --- )												        \\
// Description : A letter proposition.											        \\
//    Callers : Hangman												        \\
// 	 Calls : WordToGuess$ WordGuessed$ gnDeathLevel gnMaxDeathLevel DrawWordToGuess AskLetter AnalyseLetter		        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Game
   begin
       DrawWordToGuess
       AskLetter					       // ( --- c )
       AnalyseLetter					       // ( c --- )
       gnDeathLevel @ gnMaxDeathLevel ==		       // ( --- f )
       WordGuessed$ WordToGuess$ $==			       // ( f1 --- f1 f2 )
       or						       // ( f1 f2 --- f3 )
   until
   ;   // Game


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Diagnostic												        \\
// 	 Stack : ( --- )												        \\
// Description : Print a game diagnostic.										        \\
//    Callers : Hangman												        \\
// 	 Calls : WordToGuess$:2 WordGuessed$										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Diagnostic
   WordGuessed$ WordToGuess$ $== if			       // ( --- f )
       language @ case
	   french   of  "  F\82licitations !"   endof
		        "  Congratulations ! "
       endcase
       0 12 rot !xystring
   else
       language @ case
	   french   of  "  Le mot \82tait: "   endof
		        "  The word was: "
       endcase
       0 12 rot !xystring
       WordToGuess$ type
   endif
   ;   // Diagnostic


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : AnotherGame?												        \\
// 	 Stack : ( --- f )												        \\
// Description : Ask if an other game is wanted ? Return TRUE is yes							        \\
//    Callers : Hangman												        \\
// 	 Calls : KEY_CHANGE_LANG Yes? YesNo? Frame									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  AnotherGame?
   begin
       language @ case
	   french  of  "  Une autre partie ? "   endof
		       "  Do you want to play again ? "
       endcase
       0 14 rot !xystring				       // Show text
       key charuppercase const k0			       // ( --- c )
       k0 KEY_CHANGE_LANG == if
	   1 language @ - language !
	   frame
	   false
       else
	   k0 YesNo?					       // ( c --- f )
       endif
   until
   k0 emit
   0 14 50 0x20 !xynchar				       // Clear this line
   0 12 50 0x20 !xynchar				       // Clear the line where diagnostic was shown
   k0 Yes?						       // ( c --- f )
   ;   // AnotherGame?


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Hangman												        \\
// 	 Stack : ( --- )												        \\
// Description : The top word of the application.									        \\
//    Callers :													        \\
// 	 Calls : C_NormalText InitGame Frame Game Diagnostic AnotherGame?						        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  Hangman
   GetVideoMode const nPrevVideoMode			       // Save previous video mode
   3 SetVideoMode					       // 25x80x16, text mode, VGA
   C_NormalText ScreenBackground !			       // The screenbackground is Black
   ?UpdateCursor true!					       // Update cursor when drawing
   begin
       InitGame						       // Initialize a new game
       Frame						       // Draw the 'frame' of the game
       Game						       // Play a game
       Diagnostic					       // Print a game diagnostic
       AnotherGame? not					       // Ask if an other game ?
   until
   GetVideoMode nPrevVideoMode != if
       nPrevVideoMode SetVideoMode			       // ( n --- )
   endif
   ;   // Hangman


( Print informations about Code/Data size )
( --------------------------------------- )

dust
infos


( Uncomment next section to build stand-alone application for MS/DOS (.exe) )
( ------------------------------------------------------------------------- )

// " \\froth\\examples\\hangman.exe" save.exe
// cr " HANGMAN.EXE application size is " type . "  bytes" type

