// ---------------------------------------------------------------------------------------------------------------------------- \\
//															        \\
// File: RND.FTH        Author: [OS]		 Date: Sept, 6th    Version: 1.00					        \\
// ----		        ------			 ----		    -------						        \\
//															        \\
// Description: Test random generator: fill pages with a rancom character.						        \\
//															        \\
// Release:														        \\
//     2.00    Oct-02-95												        \\
//															        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\

( Force decimal for integer numeric I/O )
decimal

( Compiler strategy )
?speed true!						       // Choose speed vs. size
?safe true!						       // Run-time check stack under/overflow
?iarithoverflow true!					       // Run-time check integer arithmetic overflow
?chat true!						       // Print a dot for each compiled definition

forget <task>
: <task> ;

( Make SCREEN the context vocabulary )
screen

( Colors used )
( ----------- )

black LightGray BackForeGround const C_screen		       // Frame around the game


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : array													        \\
// 	 Stack : Compile-time: ( x y --- )	 x is the nb of columns, y the nb of rows				        \\
// 		     Run-time: ( x y --- addr )										        \\
// Description : Build an array of 2 dimensions.									        \\
//    Callers : done													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: array	 ( x y --- )
  build>	   \ Compile-Time
      over * WordSize + ballot				       ( x y --- x addr )
      !							       ( x addr --- )
  does>		   \ Run-Time
      dup @						       ( x y addr --- x y addr #cols )
      rot						       ( x y addr #cols --- x addr #cols y )
      *							       ( x addr #cols y --- x addr y*#cols )
      + + WordSize +					       ( x addr1 y*cols --- addr2 )
      ;

// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : done													        \\
// 	 Stack : ( x y --- addr )											        \\
// Description : Array of flags for each (col, row) to see if this screen cells has been touched.			        \\
//    Callers : Erase_done FillPage:2 SeeRnd:2										        \\
// 	 Calls : array													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
dynamic array done


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Erase_done												        \\
// 	 Stack : ( --- )												        \\
// Description : Store FALSE value in each cell of done array.								        \\
//    Callers : SeeRnd													        \\
// 	 Calls : done													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: Erase_done
  0 #cols 1- make
      0 #rows 1- make
	  0  J I done  C!
      endmake
  endmake
  ;  // Erase_done


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : constchar												        \\
// 	 Stack : Compile-time: ( c --- )       c is the character.							        \\
// 		     Run-time: ( --- c )										        \\
// Description : Defining-word to build constant character.								        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: constchar
  build>
      1 ballot
      C!
  does>
      C@
  ;  // constchar


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : FillPage												        \\
// 	 Stack : ( c --- )												        \\
// Description : Fill a complete page, rando;ly, with given character.							        \\
//    Callers : SeeRnd													        \\
// 	 Calls : done:2													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: FillPage
  constchar char
  24  #rows 1-  char  !xychar
  int counter
  #cols #rows 1- *  counter  !
  begin
      counter @ 200 mod 0== if yield endif
      0 #cols 1- rnd   0 #rows 2 - rnd			       // ( --- x y )
      ddup done c@ if
	  ddrop
      else
	  ddup char  !xychar				       // ( x y x y --- x y )
	  done 1 swap c!
	  counter 1-!
      endif
      counter @0==  ?key  or
  until
  ;  // FillPage


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : .00													        \\
// 	 Stack : ( n --- )												        \\
// Description : Print a number with 2 digits.										        \\
//    Callers : ShowInformations:3											        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: .00
  <# # # #> type
   ;  // .00


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ShowInformations											        \\
// 	 Stack : ( --- )												        \\
// Description : This is a thread: show time.										        \\
//    Callers :													        \\
// 	 Calls : .00:3													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: ShowInformations
  begin
      6 #rows 1-  gotoxy
      gettime  .00  ': emit  .00  ': emit  .00  drop
      yield
  again
  ; // ShowInformations


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : SeeRnd													        \\
// 	 Stack : ( --- )												        \\
// Description : Test and see in actions random generator.								        \\
//    Callers :													        \\
// 	 Calls : C_screen done:2 Erase_done FillPage									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: SeeRnd
  C_screen ScreenBackground !				       // The screenbackground is Black
  cls							       // Clear screen
  off cursor
   0  #rows 1-  " Time:" !xystring
  18  #rows 1-  " Char:" !xystring
  0 NewThread ShowInformations	 drop
  #cols #rows alloc done				       // Need array
  begin
      Erase_done
      0 255 rnd	 FillPage				       // Choose a random character, then fill ramdomly a page
      yield
      ?key
  until
  key drop						       // Discard character hit
  on cursor
  free done						       // Array is no more needed
  ;  // SeeRnd


( Print informations about Code/Data size )
( --------------------------------------- )

Infos

