// ---------------------------------------------------------------------------------------------------------------------------- \\
//															        \\
// File: Threads.fth    Author: Oliver SINGLA	 Date: 09-10-94	    Version: 1.00					        \\
// ----		        ------			 ----		    -------						        \\
//															        \\
// Description:														        \\
//     Demonstration program for using threads in FROTH programs.							        \\
//															        \\
// Release:														        \\
//     2.00    Oct-02-95												        \\
//															        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\

( Force decimal base for integer numeric I/O )
decimal

( Compiler strategy )
?speed true!						       // Choose speed vs. size
?safe true!						       // Run-time check stack under/overflow
?iarithoverflow true!					       // Run-time check integer arithmetic overflow
?chat true!						       // Print a dot for each compiled definition

( Empty word: this is the 1st word, we kill on each load )
forget <task>
: <task> ;


// Make SCREEN the context vocabulary
// ----------------------------------

screen


// Colors used
// -----------

black LightGray BackForeGround const C_NormalText	       ( Normal text )


// Constants positions
// -------------------

 0 const X_ActionsFrame					       ( # col )
 4 const Y_ActionsFrame					       ( # row )
27 const X_InfosFrame					       ( # col )
 4 const Y_InfosFrame					       ( # row )


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : t_integer												        \\
// 	 Stack : Building word to create private integers for each thread.						        \\
// Description : Compile time: ( --- )	  Run time: ( n --- addr )  n:thread nb	 addr:private integer address		        \\
//    Callers : threads_counters											        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: t_integer
  build>
      MaxNbThreads allot drop
  does>
      swap WordSize* +
  ; // t_integer

t_integer threads_counters


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawTittle												        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the tittle on screen										        \\
//    Callers : Frame													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: DrawTittle
  24 0 " Ŀ" !xystring
  24 1 "  FROTH THREADS EXAMPLE - [OS] " !xystring
  24 2 " " !xystring
  ;  ( DrawTittle )


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawActionsFrame											        \\
// 	 Stack : ( --- )												        \\
// Description : Draw a box which contains instructions (user interface).						        \\
//    Callers : Frame													        \\
// 	 Calls : X_ActionsFrame Y_ActionsFrame										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: DrawActionsFrame
  '~ >xylstring_char
  0 0 white     >xylstring_attr
  1 0 lightgray >xylstring_attr
  X_ActionsFrame  Y_ActionsFrame
  ddup	  " ~{1}Ŀ"         !xylstring
  1+ ddup " ~{1} ~{0}+~{1}   create a new thread " !xylstring
  1+ ddup " ~{1} ~{0}-~{1}   delete a thread     " !xylstring
  1+ ddup " ~{1} ~{0}ESC~{1} exit                " !xylstring
  1+	  " ~{1}"         !xylstring
  ; ( DrawActionsFrame )


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawInfosFrame												        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the frame for threads output and informations.							        \\
//    Callers : Frame													        \\
// 	 Calls : X_InfosFrame Y_InfosFrame										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: DrawInfosFrame
  X_InfosFrame Y_InfosFrame
  ddup	  " Ŀ" !xystring
  1+ ddup "  Nb of thread running:                             " !xystring
  1+ ddup "                                                    " !xystring
  1  MaxNbThreads 2 /  make
      1+ ddup "  Thread #" !xystring
      I <# # # #>		      !string
      " : not running"                !string
      I	 MaxNbThreads 2 /  !=  if
	  "    Thread #"                   !string
	  I MaxNbThreads 2 / + <# # # #>   !string
	  " : not running "               !string
      else
	  "                            "  !string
      endif
  endmake
  1+	  " " !xystring
  ; ( DrawInfosFrame )


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Frame													        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the frame of the application.									        \\
//    Callers : Monitor												        \\
// 	 Calls : DrawTittle DrawActionsFrame DrawInfosFrame								        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: Frame
  DrawTittle						       ( Draw tittle application )
  DrawActionsFrame					       ( Draw actions frame )
  DrawInfosFrame					       ( Draw thread informations thread )
  ; ( Frame )


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ShowThreadsInfos											        \\
// 	 Stack : ( --- )												        \\
// Description : Show threads informations										        \\
//    Callers : WaitAction												        \\
// 	 Calls : X_InfosFrame Y_InfosFrame										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: ShowThreadsInfos
  X_InfosFrame 24 +  Y_InfosFrame 1+  gotoXY  2 NbThreads 1- .R
  ;  // ShowThreadsInfos


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : WaitAction												        \\
// 	 Stack : ( --- n )												        \\
// Description : Get user action (from keyboard).									        \\
//    Callers : Monitor												        \\
// 	 Calls : X_ActionsFrame Y_ActionsFrame ShowThreadsInfos								        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: WaitAction ( --- n )
  -1							       ( --- PrevNbThreads )
  begin
      dup NbThreads != if
	  ShowThreadsInfos
	  drop NbThreads
	  X_ActionsFrame 24 +  Y_ActionsFrame 3 +  gotoXY      ( Set cursor where prompt )
      endif
      ?key						       // ( --- f ) Is a key hit ?
      yield						       ( Yield to other living threads )
  until
  drop							       ( PrevNbThreads --- )
  key							       ( --- c )
  ;  ( WaitAction )


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ThreadOutputPosition											        \\
// 	 Stack : ( n --- x y )												        \\
// Description : Compute thread output position.									        \\
//    Callers : EachThread CreateNewThread DeleteAThread								        \\
// 	 Calls : X_InfosFrame:2 Y_InfosFrame:2										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: ThreadOutputPosition
  dup  1- MaxNbThreads 2/  <  if
      X_InfosFrame 14 +					       ( --- x )
      Y_InfosFrame 2+ rot +				       ( x --- x y )
  else
      X_InfosFrame 40 +					       ( --- x )
      Y_InfosFrame 2+ rot MaxNbThreads 2/ - +		       ( x --- x y )
  endif
  ;  // ThreadOutputPosition


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : EachThread												        \\
// 	 Stack : ( --- )												        \\
// Description : Thread word.												        \\
//    Callers :													        \\
// 	 Calls : threads_counters ThreadOutputPosition									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: EachThread
  ?UpdateCursor false!					       // Don't update cursor after printing on screen
  ActiveThread ThreadOutputPosition			       // ( n --- n x y )
  ActiveThread threads_counters >R  R 0!		       // Private thread counter
  begin
      ddup  R @	 <# #S #>  !xystring
      R 1+!
      yield						       // Yield to another living thread
  again
  ;  // EachThread


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : CreateNewThread											        \\
// 	 Stack : ( --- )												        \\
// Description : Create a new thread, if possible.									        \\
//    Callers : Monitor												        \\
// 	 Calls : C_NormalText ThreadOutputPosition									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: CreateNewThread
  NbThreads  MaxNbThreads  !=  if
      0 NewThread EachThread				       // Create the new thread
      ThreadOutputPosition				       // ( n --- x y )
      C_NormalText "            " !xyastring                   // Delete the text 'Not running'
  endif
  ;   // CreateNewThread


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ReadThreadNb												        \\
// 	 Stack : ( --- ff ) if no nb, ( --- n tf ) if a thread nb has been given.					        \\
// Description : Read a thread nb.											        \\
//    Callers : DeleteAThread												        \\
// 	 Calls : X_ActionsFrame:2 Y_ActionsFrame:2									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: ReadThreadNb
  int nb      nb 0!					       // Nb read

  X_ActionsFrame 23 +  Y_ActionsFrame 2 +  gotoXY	       // Set cursor where prompt )

  logical bAbort  bAbort false!				       // Abort with ESC ?
  logical bOk	  bOk false!				       // Valide with CR ?

  begin
      key dup case
	  27 of	 bAbort true!  drop  endof
	  13 of	 bOk true!     drop  endof
	  '0 -  dup					       // Convert ascii char to digit
	  nb @  10 < if
	      nb @ if 0 else 1 endif  9	 >=<=  if	       // ( n --- n f ) Correct digit ?
		  dup '0 + emit				       // Show digit
		  10 nb *!  nb +!
	      else
		 drop
	     endif
	 else
	     ddrop
	 endif
      endcase
      bAbort true?  bOk true?  OR
  until

  ( Clear input field )
  X_ActionsFrame 23 +  Y_ActionsFrame 2 +  "   " !xystring

  ( Return value/flag )
  bOk true?  nb @ 0 MaxNbThreads >=<  AND if
      nb @  TRUE
  else
      FALSE
  endif

  ;  // ReadThreadNb


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DeleteAThread												        \\
// 	 Stack : ( --- )												        \\
// Description : Ask for a thread (nb), then kill this thread.								        \\
//    Callers : Monitor												        \\
// 	 Calls : ThreadOutputPosition ReadThreadNb									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: DeleteAThread
  ReadThreadNb						       // ( --- ff ) or ( --- n tf )
  if
      dup KillThread					       // ( n --- f )
      ThreadOutputPosition				       // ( n --- n x y )
      " not running"  !xystring
  endif
  ;  // DeleteAThread


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Monitor												        \\
// 	 Stack : ( --- )												        \\
// Description : The top word of the application.									        \\
//    Callers : g													        \\
// 	 Calls : C_NormalText Frame WaitAction CreateNewThread DeleteAThread						        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: Monitor
  GetVideoMode						       // ( --- n )
  3 SetVideoMode					       // 25x80x16, text mode, VGA
  C_NormalText ScreenBackground !			       // The screenbackground is Black
  cls							       // Clear screen
  ?UpdateCursor true!					       // Update cursor when printing on screen
  Frame							       // Draw the 'frame' of the application
  logical bStop	   bStop false!				       // Local variable: exit loop ?
  begin
      WaitAction					       // Get keyboard action
      fkeys						       // Functions keys vocabulary
      case
	  KEY_ESC  of bStop true!     endof		       // <ESC> -> End application
	  KEY_+	   of CreateNewThread endof		       // <+>   -> Create a new thread, if possible
	  KEY_-	   of DeleteAThread   endof		       // <->   -> Delete a given thread, if active
      endcase
      screen						       // Screen vocabulary
      bStop true?
  until
  SetVideoMode						       // ( n --- )
  ;  // Monitor

: g monitor ;


( Show infos about code/data size )
( ------------------------------- )

infos

