// ---------------------------------------------------------------------------------------------------------------------------- \\
//															        \\
// File: THRESHOL.FTH   Author: Oliver SINGLA	 Date: Nov, 30, 94  Version: 0.02					        \\
// ----		        ------			 ----		    -------						        \\
//															        \\
// Description:	 Threshold Arcade Game.											        \\
//															        \\
// Note: Well, this is the just the beginning. Nevertheless, I put on disk because it shows using many graphics functions.      \\
//															        \\
// (This game (and WinThreshold) was inspired from a game which ran on apple ][ by Ken Williams).			        \\
//															        \\
// Release:														        \\
//     0.03    Oct-02-95												        \\
//															        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\

( Force decimal for integer numeric I/O )
decimal

( Compiler strategy )
?speed true!						       // Choose speed vs. size
?safe true!						       // Run-time check stack under/overflow
?iarithoverflow true!					       // Run-time check integer arithmetic overflow
?chat true!						       // Print a dot for each compiled definition

( Make GRAPHICS the context vocabulary )
graphics

( Compilation must be done under graphics mode, this is because sprite are created here )
640x350x16
screen 0 >page graphics
0 clear
" Compiling in action..." type

( Empty word: this is the 1st word, we kill on each load )
forget <task>
: <task> ;

( Size of a text character )
dynamic const char_height
dynamic const char_width

( Fuel gauge )
dynamic const FuelGauge_X
dynamic const FuelGauge_Y
dynamic const FuelGauge_height
dynamic const FuelGauge_width
screen
yellow	     const C_FuelGauge_label
lightgray    const C_FuelGauge_border
lightgreen   const C_FuelGauge_ok
lightred     const C_FuelGauge_nok
graphics

( Laser gauge )
dynamic const LaserGauge_X
dynamic const LaserGauge_Y
dynamic const LaserGauge_height
dynamic const LaserGauge_width
screen
yellow	     const C_LaserGauge_label
lightgray    const C_LaserGauge_border
lightgreen   const C_LaserGauge_ok
lightred     const C_LaserGauge_nok
graphics

( Game's universe )
dynamic const UniverseBorder_height
dynamic const UniverseBorder_width
screen
lightgray    const C_UniverseBorder
black	     const C_UniverseBackground
graphics
dynamic const Universe_TopLeft_X
dynamic const Universe_TopLeft_Y
dynamic const Universe_TopRight_X
dynamic const Universe_TopRight_Y
dynamic const Universe_BottomLeft_X
dynamic const Universe_BottomLeft_Y
dynamic const Universe_BottomRight_X
dynamic const Universe_BottomRight_Y


( --- )
( Gun )
( --- )

    20 const gun_xsize
    20 const gun_ysize
    int gun_X						       // X-position for the gun
    dynamic const Gun_Y					       // Y-position for the gun
    3 const GUN_XINC					       // X-increment when gun moving
    int gun_xmin					       // Left  limit
    int gun_xmax					       // Right limit
    fkeys
    KEY_LEFT  const MOVE_GUN_LEFT			       // Key action to move gun to left
    int MoveGunLeft_KeyIndice				       // Indice for the key to move gun left
    KEY_RIGHT const MOVE_GUN_RIGHT			       // Key action to move gun to right
    int MoveGunRight_KeyIndice				       // Indice for the key to move gun right
    'Z	      const SHOOT_GUN				       // Key action to shoot gun
    int ShootGun_KeyIndice				       // Indice for the key to shoot gun
    graphics


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : points_array												        \\
// 	 Stack :													        \\
// Description :													        \\
//    Callers : 'T__xy 'H1_xy 'R__xy 'E__xy 'S__xy 'H2_xy 'O__xy 'L__xy 'D__xy						        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  points_array
   build>
       2* allot
       drop
   does>
       swap WordSize* 2* +
   ;   // points_array


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : carray													        \\
// 	 Stack : Compile-time: ( n --- ) <n> is the size of the bytes array						        \\
// 		     Run-time: ( n --- ) <n> is the indice into the array						        \\
// Description : One-dimension bytes array.										        \\
//    Callers : prelude_AnimTHR_tbl											        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: carray
   build>
       ballot						       // ( n --- addr )
       drop						       // ( addr --- )
   does>
       +						       // ( n addr --- n+addr )
   ;   // array


:  des_ships>
   build>
       1 allot  !
   does>
       swap WordSize* 4 * +
   ;   // des_ships


:  ,
   1 AllotMore
   !
   ;   // ,


" thr_spru" "load                                              // Sprites used
" thr_thrl" "load                                              // 'THRESHOLD' letters


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : !gxyastring												        \\
// 	 Stack : ( x y attr addrz --- )											        \\
// Description : Print a string at (x, y), which are graphics (and not text screen) coordonnates.			        \\
//    Callers : InstallFuelGauge InstallLaserGauge									        \\
// 	 Calls : char_height char_width											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  !gxyastring ( x y a addrz --- )
   dswap
   swap char_width / swap
   char_height /
   dswap
   screen !xyastring graphics
   ;   // gxyastring


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : 2pages?												        \\
// 	 Stack : ( --- f )												        \\
// Description : Is there at least two video pages available ?								        \\
//    Callers : DrawFrame Prelude_frame Prelude_AnimThreshold Prelude_gun						        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  2pages?
   screen #pages 1 > graphics
   ;   // 2pages?


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ShowPage0												        \\
// 	 Stack : ( --- )												        \\
// Description : Show graphics page #0											        \\
//    Callers : Prelude_AnimThreshold threshold									        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ShowPage0
   screen 0 >page graphics
   ;   // ShowPage0


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ShowPage1												        \\
// 	 Stack : ( --- )												        \\
// Description : Show graphics page #1											        \\
//    Callers : Prelude_AnimThreshold											        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ShowPage1
   screen 1 >page graphics
   ;   // ShowPage1


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ShowPage~												        \\
// 	 Stack : ( --- )												        \\
// Description : Show graphics page not current.									        \\
//    Callers : Prelude_gun												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  ShowPage~
   screen 1 page# @ - >page graphics
   ;   // ShowPage~


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : SelectPage0												        \\
// 	 Stack : ( --- )												        \\
// Description : Select graphics page #0										        \\
//    Callers : Prelude_AnimThreshold											        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  SelectPage0
   0 SelectGPage
   ;   // SelectPage0


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : SelectPage1												        \\
// 	 Stack : ( --- )												        \\
// Description : Select graphics page #1										        \\
//    Callers : Prelude_AnimThreshold											        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  SelectPage1
   1 SelectGPage
   ;   // SelectPage1


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : SelectPage~												        \\
// 	 Stack : ( --- )												        \\
// Description : Select graphics page not current.									        \\
//    Callers : Prelude_gun:2												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  SelectPage~
   screen 1 page# @ - graphics SelectGPage
   ;   // SelectPage~


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : InitFramePosAndSize											        \\
// 	 Stack : ( --- )												        \\
// Description : Initialize the position and the size of all STATIC elements of the games (FRAME).			        \\
//    Callers : threshold												        \\
// 	 Calls : char_height char_width:2 FuelGauge_X FuelGauge_Y:2 FuelGauge_height FuelGauge_width LaserGauge_X LaserGauge_Y  \\
// 		 LaserGauge_height LaserGauge_width UniverseBorder_height:2 UniverseBorder_width Universe_TopLeft_X:2	        \\
// 		 Universe_TopLeft_Y:2 Universe_TopRight_X:2 Universe_TopRight_Y Universe_BottomLeft_X:2 Universe_BottomLeft_Y:2 \\
// 		 Universe_BottomRight_X Universe_BottomRight_Y Gun_Y gun_xmin gun_xmax MOVE_GUN_LEFT MoveGunLeft_KeyIndice      \\
// 		 MOVE_GUN_RIGHT MoveGunRight_KeyIndice SHOOT_GUN ShootGun_KeyIndice					        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: InitFramePosAndSize

  ( Size of a text character )
  { #grows screen / #rows graphics } alloc char_height
  { #gcols screen / #cols graphics } alloc char_width

  ( Fuel gauge )
  { char_width * 2 }					        alloc FuelGauge_width
  { #grows - ( ( char_height * 3 ) + ( 3 * 4 ) ) }	        alloc FuelGauge_height
  char_width						        alloc FuelGauge_X
  { char_height + 3 }					        alloc FuelGauge_Y

  ( Laser gauge )
  { char_width * 2 }					        alloc LaserGauge_width
  { #grows - ( ( char_height * 3 ) + ( 3 * 4 ) ) }	        alloc LaserGauge_height
  { #gcols - ( char_width * 5 ) }			        alloc LaserGauge_X
  FuelGauge_Y						        alloc LaserGauge_Y

  ( Game's universe )
  6							        alloc UniverseBorder_height
  6							        alloc UniverseBorder_width
  { FuelGauge_X + FuelGauge_width + 12 + UniverseBorder_width } alloc Universe_TopLeft_X
  UniverseBorder_height					        alloc Universe_TopLeft_Y
  { ( LaserGauge_X - 12 ) - UniverseBorder_width }	        alloc Universe_TopRight_X
  Universe_TopLeft_Y					        alloc Universe_TopRight_Y
  Universe_TopLeft_X					        alloc Universe_BottomLeft_X
  { #grows - 1 }					        alloc Universe_BottomLeft_Y
  Universe_TopRight_X					        alloc Universe_BottomRight_X
  Universe_BottomLeft_Y					        alloc Universe_BottomRight_Y

  ( Gun )
  alloc { #grows - gun_ysize } gun_y			       // Init y-position for the gun (never change)
  Universe_BottomLeft_X 1+  gun_xmin !			       // Left  limit
  { ( Universe_bottomRight_X - gun_xsize ) - 1 } gun_xmax !    // Right limit
  fkeys
  MOVE_GUN_LEFT	 KeyIndice MoveGunLeft_KeyIndice !	       // Move gun to left
  MOVE_GUN_RIGHT KeyIndice MoveGunRight_KeyIndice !	       // Move gun to right
  SHOOT_GUN	 KeyIndice ShootGun_KeyIndice !		       // Key action to shoot gun
  graphics

  ;  // InitFramePosAndSize


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : FreeDynaVars												        \\
// 	 Stack : ( --- )												        \\
// Description : At the end of the application, free *all* dynamic variables (clean).					        \\
//    Callers :													        \\
// 	 Calls : char_height char_width FuelGauge_X FuelGauge_Y FuelGauge_height FuelGauge_width LaserGauge_X LaserGauge_Y      \\
// 		 LaserGauge_height LaserGauge_width UniverseBorder_height UniverseBorder_width				        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: FreeDynaVars

  ( Size of a text character )
  free char_height
  free char_width

  ( Fuel gauge )
  free FuelGauge_X
  free FuelGauge_Y
  free FuelGauge_height
  free FuelGauge_width

  ( Laser gauge )
  free LaserGauge_X
  free LaserGauge_Y
  free LaserGauge_height
  free LaserGauge_width

  ( Game's universe )
  free UniverseBorder_height
  free UniverseBorder_width

  ;  // FreeDynaVars


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : InstallFuelGauge											        \\
// 	 Stack : ( --- )												        \\
// Description : Install the fuel gauge.										        \\
//    Callers : DrawFrame												        \\
// 	 Calls : char_height char_width FuelGauge_X:3 FuelGauge_Y:3 FuelGauge_height:2 FuelGauge_width C_FuelGauge_label        \\
// 		 C_FuelGauge_border:2 !gxyastring									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: InstallFuelGauge

  FuelGauge_X FuelGauge_Y dup FuelGauge_height + C_FuelGauge_border vline
  FuelGauge_X FuelGauge_width + FuelGauge_Y dup FuelGauge_height + C_FuelGauge_border vline

  screen
  FuelGauge_X char_width - FuelGauge_Y char_height - 3 -  C_FuelGauge_label  " FUEL"  !gxyastring
  graphics

  ;  // InstallFuelGauge


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : InstallLaserGauge											        \\
// 	 Stack : ( --- )												        \\
// Description : Install the laser gauge.										        \\
//    Callers : DrawFrame												        \\
// 	 Calls : char_height char_width LaserGauge_X:3 LaserGauge_Y:3 LaserGauge_height:2 LaserGauge_width C_LaserGauge_label   \\
// 		 C_LaserGauge_border:2 !gxyastring									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: InstallLaserGauge

  LaserGauge_X LaserGauge_Y dup LaserGauge_height + C_LaserGauge_border vline
  LaserGauge_X LaserGauge_width + LaserGauge_Y dup LaserGauge_height + C_LaserGauge_border vline

  screen
  LaserGauge_X char_width - LaserGauge_Y char_height - 3 -  C_LaserGauge_label  " LASER"  !gxyastring
  graphics

  ;  // InstallLaserGauge


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : InstallUniverse'sGame											        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the universe's game.										        \\
//    Callers : DrawFrame												        \\
// 	 Calls : UniverseBorder_height:2 UniverseBorder_width:4 C_UniverseBorder:3 Universe_TopLeft_X:2 Universe_TopLeft_Y:2    \\
// 		 Universe_TopRight_X:2 Universe_TopRight_Y:2 Universe_BottomLeft_X Universe_BottomLeft_Y Universe_BottomRight_X \\
// 		 Universe_BottomRight_Y											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: InstallUniverse'sGame

  Universe_TopLeft_X   UniverseBorder_width  -
  Universe_TopLeft_Y   UniverseBorder_height -
  Universe_TopRight_X  UniverseBorder_width  +
  Universe_TopRight_Y  1-
  C_UniverseBorder FILLED box

  Universe_TopLeft_X   UniverseBorder_width  -
  Universe_TopLeft_Y   UniverseBorder_height -
  Universe_BottomLeft_X 1-
  Universe_BottomLeft_Y
  C_UniverseBorder FILLED box

  Universe_TopRight_X 1+
  Universe_TopRight_Y
  Universe_BottomRight_X UniverseBorder_width + 1-
  Universe_BottomRight_Y
  C_UniverseBorder FILLED box

  ;  // InstallUniverse'sGame


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : InitSprites												        \\
// 	 Stack : ( --- )												        \\
// Description : Initialize all sprites used.										        \\
//    Callers : DrawFrame												        \\
// 	 Calls : s_ArmedGun s_Alien_0 s_Alien_1 s_Alien_2 s_Alien_3 s_Alien_4 s_Alien_5 s_Alien_6 s_threshold_'T'	        \\
// 		 s_threshold_'R' s_threshold_'O' s_threshold_'S' s_threshold_'H' s_threshold_'E' s_threshold_'L'	        \\
// 		 s_threshold_'D'											        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: InitSprites

  s_threshold_'T'       Init_Sprite			       // 'THRESHOLD' : 'T' letter
  s_threshold_'H'       Init_Sprite			       // 'THRESHOLD' : 'H' letter
  s_threshold_'R'       Init_Sprite			       // 'THRESHOLD' : 'R' letter
  s_threshold_'E'       Init_Sprite			       // 'THRESHOLD' : 'E' letter
  s_threshold_'S'       Init_Sprite			       // 'THRESHOLD' : 'S' letter
  s_threshold_'O'       Init_Sprite			       // 'THRESHOLD' : 'O' letter
  s_threshold_'L'       Init_Sprite			       // 'THRESHOLD' : 'L' letter
  s_threshold_'D'       Init_Sprite			       // 'THRESHOLD' : 'D' letter

  s_ArmedGun	        Init_Sprite			       // Armed gun

  s_Alien_0	        Init_Sprite			       // Alien, state #0
  s_Alien_1	        Init_Sprite			       // Alien, state #1
  s_Alien_2	        Init_Sprite			       // Alien, state #2
  s_Alien_3	        Init_Sprite			       // Alien, state #3
  s_Alien_4	        Init_Sprite			       // Alien, state #4
  s_Alien_5	        Init_Sprite			       // Alien, state #5
  s_Alien_6	        Init_Sprite			       // Alien, state #6

  ;  // InitSprites


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawFrame												        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the frame of the game in the current page.							        \\
//    Callers : threshold												        \\
// 	 Calls : 2pages? InstallFuelGauge InstallLaserGauge InstallUniverse'sGame InitSprites				        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: DrawFrame
  0 clear						       // Clear graphics screen
  InitSprites						       // Initialize all sprites
  2pages? if  0 1  else	 0 0  endif
  make
      I screen >page graphics				       // Draw into the two video pages (if any)
      InstallFuelGauge					       // Draw the fuel gauge
      InstallLaserGauge					       // Draw the laser gauge
      InstallUniverse'sGame				       // Draw the game's universe
  endmake
  ;  // DrawFrame


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : xy!													        \\
// 	 Stack : ( x y addr --- )											        \\
// Description : Store a point (x, y).											        \\
//    Callers : Prelude_AnimTHR_init:H3										        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  xy!

   ( Compute the position of the center letter 'S' )
   { Universe_TopLeft_X + ( ( ( Universe_TopRight_X - Universe_TopLeft_X ) - 15 ) / 2 ) } const x0
   30 const y0

   swap over WordSize +					       // ( x y addr --- x addr y addr+4 )
   swap y0 + swap !					       // ( x addr y addr+4 --- x addr )
   swap x0 + swap !					       // ( x addr --- )
   ;   // xy!


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : xy@													        \\
// 	 Stack : ( addr --- x y )											        \\
// Description : Read a point from addr.										        \\
//    Callers : draw_'THRESHOLD':9											        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  xy@
   dup @						       // ( addr --- addr x )
   swap WordSize + @					       // ( addr x --- x y )
   ;   // xy@


dynamic points_array 'T__xy
dynamic points_array 'H1_xy
dynamic points_array 'R__xy
dynamic points_array 'E__xy
dynamic points_array 'S__xy
dynamic points_array 'H2_xy
dynamic points_array 'O__xy
dynamic points_array 'L__xy
dynamic points_array 'D__xy


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Prelude_frame_menu											        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the menu (keys available) on prelude page.							        \\
//    Callers : Prelude_frame												        \\
// 	 Calls : char_height:4 char_width:6 Universe_TopLeft_X:2 Universe_TopRight_X					        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: Prelude_frame_menu

  26 const xsize					    // X size
   6 const ysize					    // Y size
  Universe_TopRight_X Universe_TopLeft_X -  xsize char_width *  -  2/
  Universe_TopLeft_X +  char_width /
     const x0						    // X position
  6  const y0						    // Y position
  screen
  darkgray const background
  x0 char_width	 * char_width  2/ -
  y0 char_height * char_height 4 / -
  over xsize char_width	 * + char_width	 +
  over ysize char_height * + char_height 2 / +
  background graphics FILLED box screen
  '~ >xylstring_char
  0 background lightgray >xylstring_attr
  1 background red	 >xylstring_attr
  2 background lightred	 >xylstring_attr
  3 background lightgreen >xylstring_attr
  x0  y0 0 +  " ~{2}P~{0}  Play game"                   !xylstring
  x0  y0 1 +  " ~{2}S~{0}  Sound on/off"                !xylstring
  x0  y0 2 +  " ~{2}K~{0}  Use the keyboard"            !xylstring
  x0  y0 3 +  " ~{2}J~{0}  Use the joystick"            !xylstring
  x0  y0 4 +  " ~{2}H~{0}  High scores"                 !xylstring
  x0  y0 5 +  " ~{1}X~{0}  Exit ~{3}THRESHOLD v0.01.a"  !xylstring
  graphics

  ;  // Prelude_frame_menu


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Prelude_frame_infos											        \\
// 	 Stack : ( --- )												        \\
// Description : Draw some infos on prelude page.									        \\
//    Callers : Prelude_frame												        \\
// 	 Calls : char_height:4 char_width:6 Universe_TopLeft_X:2 Universe_TopRight_X					        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: Prelude_frame_infos

  64 const xsize					    // X size
   9 const ysize					    // Y size
  Universe_TopRight_X Universe_TopLeft_X -  xsize char_width *  -  2/
  Universe_TopLeft_X +  char_width /
     const x0						    // X position
  13 const y0						    // Y position
  screen
  darkgray const background
  x0 char_width	 * char_width  2/ -
  y0 char_height * char_height 4 / -
  over xsize char_width	 * + char_width	 +
  over ysize char_height * + char_height 2/ +
  background graphics FILLED box screen
  '~ >xylstring_char
  0 background lightgray >xylstring_attr
  1 background red	 >xylstring_attr
  2 background brown	 >xylstring_attr
  3 background cyan	 >xylstring_attr
  4 background white	 >xylstring_attr
  x0  y0 0 +  " ~{0}This game is written in ~{1}FROTH~{0}.  This is a free implementation of"                   !xylstring
  x0  y0 1 +  " ~{1}FORTH~{0} language. Here are the highlights of FROTH language:"                             !xylstring
  x0  y0 2 +  " ~{4} Complete and highly interactive environment  (compiler, editor,"                           !xylstring
  x0  y0 3 +  " ~{4} interpreter) ~{2}*~{4} Run in 32-bits protected mode ~{2}*~{4} Tools for cross"            !xylstring
  x0  y0 4 +  " ~{4} reference ~{2}*~{4} Definitions  browser ~{2}*~{4} Complete online help ~{2}*~{4} Up to"   !xylstring
  x0  y0 5 +  " ~{4} 32 threads (cooperative  multi-tasking) ~{2}*~{4} Dynamic and automatic"                   !xylstring
  x0  y0 6 +  " ~{4} variables  (garbage collector) ~{2}*~{4} Ability to build .EXE files..."                   !xylstring
  x0  y0 7 +  " ~{4} Comes with a complete 100 pages manual: tutorial and reference."                           !xylstring
  x0  y0 8 +  " ~{3}--- Oliver SINGLA - FROTH v 0.25.b - singla@email.teaser.com ---"                           !xylstring
  graphics

  ;  // Prelude_frame_infos


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Prelude_frame												        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the frame for prelude.										        \\
//    Callers : Prelude												        \\
// 	 Calls : C_UniverseBackground Universe_TopLeft_X Universe_TopLeft_Y Universe_BottomRight_X Universe_BottomRight_Y       \\
// 		 gun_X:2 Gun_Y s_ArmedGun s_Alien_0 s_Alien_1 s_Alien_2 s_Alien_3 s_Alien_4 s_Alien_5 s_Alien_6 2pages?	        \\
// 		 Prelude_frame_menu Prelude_frame_infos									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: Prelude_frame

  2pages? if  0 1  else	 0 0  endif
  make
      I screen >page graphics				       // Draw into the two video pages (if any)

      ( Clear the entire universe's game area )
      gmode_copy
      Universe_TopLeft_X      Universe_TopLeft_Y
      Universe_BottomRight_X  Universe_BottomRight_Y
      C_UniverseBackground FILLED
      box

      ( Draw the menu )
      Prelude_frame_menu

      ( Draw some infos about FROTH and this game's version )
      Prelude_frame_infos

      ( Install the gun )
      { ( ( ( Universe_BottomRight_X - Universe_BottomLeft_X ) - gun_xsize ) / 2 ) + Universe_BottomLeft_X }
      gun_x !
      gun_x @  gun_y  s_ArmedGun  draw_sprite

100  20 s_Alien_0 Draw_Sprite
100  40 s_Alien_1 Draw_Sprite
100  60 s_Alien_2 Draw_Sprite
100  80 s_Alien_3 Draw_Sprite
100 100 s_Alien_4 Draw_Sprite
100 120 s_Alien_5 Draw_Sprite
100 140 s_Alien_6 Draw_Sprite
  endmake

  ;  // Prelude_frame


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : draw_'THRESHOLD'											        \\
// 	 Stack : ( n --- )												        \\
// Description : Draw the letters: THRESHOLD										        \\
//    Callers : Prelude_AnimThreshold:5										        \\
// 	 Calls : s_threshold_'T' s_threshold_'R' s_threshold_'O' s_threshold_'S' s_threshold_'H':2 s_threshold_'E'	        \\
// 		 s_threshold_'L' s_threshold_'D' xy@:9 'T__xy 'H1_xy 'R__xy 'E__xy 'S__xy 'H2_xy 'O__xy 'L__xy 'D__xy	        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: draw_'THRESHOLD'
  dup 'T__xy xy@  s_threshold_'T' Draw_Sprite
  dup 'H1_xy xy@  s_threshold_'H' Draw_Sprite
  dup 'R__xy xy@  s_threshold_'R' Draw_Sprite
  dup 'E__xy xy@  s_threshold_'E' Draw_Sprite
  dup 'S__xy xy@  s_threshold_'S' Draw_Sprite
  dup 'H2_xy xy@  s_threshold_'H' Draw_Sprite
  dup 'O__xy xy@  s_threshold_'O' Draw_Sprite
  dup 'L__xy xy@  s_threshold_'L' Draw_Sprite
      'D__xy xy@  s_threshold_'D' Draw_Sprite
  ;  // draw_'THRESHOLD'


  27 const 'THRESHOLD'_#states				       // Number of different states (must be odd)
  int Prelude_AnimTHR_I'0
  int Prelude_AnimTHR_I'1
  'THRESHOLD'_#states dup 2- + carray prelude_AnimTHR_tbl
  int prelude_AnimTHR_index

// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Prelude_AnimTHR_init											        \\
// 	 Stack : ( --- )												        \\
// Description : While waiting user's action, screen animation.								        \\
//    Callers : Prelude												        \\
// 	 Calls : xy!:H3 'T__xy:28 'H1_xy:28 'R__xy:28 'E__xy:28 'S__xy:28 'H2_xy:28 'O__xy:28 'L__xy:28 'D__xy:28	        \\
// 		 'THRESHOLD'_#states:11 Prelude_AnimTHR_I'0 Prelude_AnimTHR_I'1 prelude_AnimTHR_tbl prelude_AnimTHR_index       \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: Prelude_AnimTHR_init

  ( Compute the position of letter 'T' )
  'THRESHOLD'_#states alloc 'T__xy
  -90
  dup  -22   0 'T__xy  xy!
  dup  -20   1 'T__xy  xy!
  dup  -18   2 'T__xy  xy!
  dup  -16   3 'T__xy  xy!
  dup  -14   4 'T__xy  xy!
  dup  -12   5 'T__xy  xy!
  dup  -10   6 'T__xy  xy!
  dup   -8   7 'T__xy  xy!
  dup   -6   8 'T__xy  xy!
  dup   -5   9 'T__xy  xy!
  dup   -3  10 'T__xy  xy!
  dup   -2  11 'T__xy  xy!
  dup   -1  12 'T__xy  xy!
  dup	 0  13 'T__xy  xy!
  dup	 1  14 'T__xy  xy!
  dup	 2  15 'T__xy  xy!
  dup	 3  16 'T__xy  xy!
  dup	 5  17 'T__xy  xy!
  dup	 6  18 'T__xy  xy!
  dup	 8  19 'T__xy  xy!
  dup   10  20 'T__xy  xy!
  dup   12  21 'T__xy  xy!
  dup   14  22 'T__xy  xy!
  dup   16  23 'T__xy  xy!
  dup   18  24 'T__xy  xy!
  dup   20  25 'T__xy  xy!
        22  26 'T__xy  xy!

  ( Compute the position of letter 'H' )
  'THRESHOLD'_#states alloc 'H1_xy
  -68
  dup  -17   0 'H1_xy  xy!
  dup  -16   1 'H1_xy  xy!
  dup  -15   2 'H1_xy  xy!
  dup  -13   3 'H1_xy  xy!
  dup  -11   4 'H1_xy  xy!
  dup   -9   5 'H1_xy  xy!
  dup   -7   6 'H1_xy  xy!
  dup   -6   7 'H1_xy  xy!
  dup   -5   8 'H1_xy  xy!
  dup   -4   9 'H1_xy  xy!
  dup   -3  10 'H1_xy  xy!
  dup   -1  11 'H1_xy  xy!
  dup   -1  12 'H1_xy  xy!
  dup	 0  13 'H1_xy  xy!
  dup	 1  14 'H1_xy  xy!
  dup	 1  15 'H1_xy  xy!
  dup	 3  16 'H1_xy  xy!
  dup	 4  17 'H1_xy  xy!
  dup	 5  18 'H1_xy  xy!
  dup	 6  19 'H1_xy  xy!
  dup	 7  20 'H1_xy  xy!
  dup	 9  21 'H1_xy  xy!
  dup   11  22 'H1_xy  xy!
  dup   13  23 'H1_xy  xy!
  dup   15  24 'H1_xy  xy!
  dup   16  25 'H1_xy  xy!
        17  26 'H1_xy  xy!

  ( Compute the position of letter 'R' )
  'THRESHOLD'_#states alloc 'R__xy
  -43
  dup  -13   0 'R__xy  xy!
  dup  -11   1 'R__xy  xy!
  dup  -11   2 'R__xy  xy!
  dup  -10   3 'R__xy  xy!
  dup   -8   4 'R__xy  xy!
  dup   -7   5 'R__xy  xy!
  dup   -6   6 'R__xy  xy!
  dup   -5   7 'R__xy  xy!
  dup   -4   8 'R__xy  xy!
  dup   -3   9 'R__xy  xy!
  dup   -2  10 'R__xy  xy!
  dup   -1  11 'R__xy  xy!
  dup   -1  12 'R__xy  xy!
  dup	 0  13 'R__xy  xy!
  dup	 1  14 'R__xy  xy!
  dup	 1  15 'R__xy  xy!
  dup	 2  16 'R__xy  xy!
  dup	 3  17 'R__xy  xy!
  dup	 4  18 'R__xy  xy!
  dup	 5  19 'R__xy  xy!
  dup	 6  20 'R__xy  xy!
  dup	 7  21 'R__xy  xy!
  dup	 8  22 'R__xy  xy!
  dup   10  23 'R__xy  xy!
  dup   11  24 'R__xy  xy!
  dup   11  25 'R__xy  xy!
        13  26 'R__xy  xy!

  ( Compute the position of letter 'E' )
  'THRESHOLD'_#states alloc 'E__xy
  -21
  dup  -6    0 'E__xy  xy!
  dup  -6    1 'E__xy  xy!
  dup  -5    2 'E__xy  xy!
  dup  -5    3 'E__xy  xy!
  dup  -4    4 'E__xy  xy!
  dup  -4    5 'E__xy  xy!
  dup  -3    6 'E__xy  xy!
  dup  -3    7 'E__xy  xy!
  dup  -2    8 'E__xy  xy!
  dup  -2    9 'E__xy  xy!
  dup  -1   10 'E__xy  xy!
  dup  -1   11 'E__xy  xy!
  dup   0   12 'E__xy  xy!
  dup   0   13 'E__xy  xy!
  dup   0   14 'E__xy  xy!
  dup   1   15 'E__xy  xy!
  dup   1   16 'E__xy  xy!
  dup   2   17 'E__xy  xy!
  dup   2   18 'E__xy  xy!
  dup   3   19 'E__xy  xy!
  dup   3   20 'E__xy  xy!
  dup   4   21 'E__xy  xy!
  dup   4   22 'E__xy  xy!
  dup   5   23 'E__xy  xy!
  dup   5   24 'E__xy  xy!
  dup   6   25 'E__xy  xy!
        6   26 'E__xy  xy!

  ( Compute the position of letter 'S' )
  'THRESHOLD'_#states alloc 'S__xy
  0
  dup	 0   0 'S__xy  xy!
  dup	 0   1 'S__xy  xy!
  dup	 0   2 'S__xy  xy!
  dup	 0   3 'S__xy  xy!
  dup	 0   4 'S__xy  xy!
  dup	 0   5 'S__xy  xy!
  dup	 0   6 'S__xy  xy!
  dup	 0   7 'S__xy  xy!
  dup	 0   8 'S__xy  xy!
  dup	 0   9 'S__xy  xy!
  dup	 0  10 'S__xy  xy!
  dup	 0  11 'S__xy  xy!
  dup	 0  12 'S__xy  xy!
  dup	 0  13 'S__xy  xy!
  dup	 0  14 'S__xy  xy!
  dup	 0  15 'S__xy  xy!
  dup	 0  16 'S__xy  xy!
  dup	 0  17 'S__xy  xy!
  dup	 0  18 'S__xy  xy!
  dup	 0  19 'S__xy  xy!
  dup	 0  20 'S__xy  xy!
  dup	 0  21 'S__xy  xy!
  dup	 0  22 'S__xy  xy!
  dup	 0  23 'S__xy  xy!
  dup	 0  24 'S__xy  xy!
  dup	 0  25 'S__xy  xy!
	 0  26 'S__xy  xy!

  ( Compute the position of letter 'H' )
  'THRESHOLD'_#states alloc 'H2_xy
  17
  dup  -6    0 'H2_xy  xy!
  dup  -6    1 'H2_xy  xy!
  dup  -5    2 'H2_xy  xy!
  dup  -5    3 'H2_xy  xy!
  dup  -4    4 'H2_xy  xy!
  dup  -4    5 'H2_xy  xy!
  dup  -3    6 'H2_xy  xy!
  dup  -3    7 'H2_xy  xy!
  dup  -2    8 'H2_xy  xy!
  dup  -2    9 'H2_xy  xy!
  dup  -1   10 'H2_xy  xy!
  dup  -1   11 'H2_xy  xy!
  dup   0   12 'H2_xy  xy!
  dup   0   13 'H2_xy  xy!
  dup   0   14 'H2_xy  xy!
  dup   1   15 'H2_xy  xy!
  dup   1   16 'H2_xy  xy!
  dup   2   17 'H2_xy  xy!
  dup   2   18 'H2_xy  xy!
  dup   3   19 'H2_xy  xy!
  dup   3   20 'H2_xy  xy!
  dup   4   21 'H2_xy  xy!
  dup   4   22 'H2_xy  xy!
  dup   5   23 'H2_xy  xy!
  dup   5   24 'H2_xy  xy!
  dup   6   25 'H2_xy  xy!
        6   26 'H2_xy  xy!

  ( Compute the position of letter 'O' )
  'THRESHOLD'_#states alloc 'O__xy
  42
  dup  -13   0 'O__xy  xy!
  dup  -11   1 'O__xy  xy!
  dup  -11   2 'O__xy  xy!
  dup  -10   3 'O__xy  xy!
  dup   -8   4 'O__xy  xy!
  dup   -7   5 'O__xy  xy!
  dup   -6   6 'O__xy  xy!
  dup   -5   7 'O__xy  xy!
  dup   -4   8 'O__xy  xy!
  dup   -3   9 'O__xy  xy!
  dup   -2  10 'O__xy  xy!
  dup   -1  11 'O__xy  xy!
  dup   -1  12 'O__xy  xy!
  dup	 0  13 'O__xy  xy!
  dup	 1  14 'O__xy  xy!
  dup	 1  15 'O__xy  xy!
  dup	 2  16 'O__xy  xy!
  dup	 3  17 'O__xy  xy!
  dup	 4  18 'O__xy  xy!
  dup	 5  19 'O__xy  xy!
  dup	 6  20 'O__xy  xy!
  dup	 7  21 'O__xy  xy!
  dup	 8  22 'O__xy  xy!
  dup   10  23 'O__xy  xy!
  dup   11  24 'O__xy  xy!
  dup   11  25 'O__xy  xy!
        13  26 'O__xy  xy!

  ( Compute the position of letter 'L' )
  'THRESHOLD'_#states alloc 'L__xy
  68
  dup  -17   0 'L__xy  xy!
  dup  -16   1 'L__xy  xy!
  dup  -15   2 'L__xy  xy!
  dup  -13   3 'L__xy  xy!
  dup  -11   4 'L__xy  xy!
  dup   -9   5 'L__xy  xy!
  dup   -7   6 'L__xy  xy!
  dup   -6   7 'L__xy  xy!
  dup   -5   8 'L__xy  xy!
  dup   -4   9 'L__xy  xy!
  dup   -3  10 'L__xy  xy!
  dup   -1  11 'L__xy  xy!
  dup   -1  12 'L__xy  xy!
  dup	 0  13 'L__xy  xy!
  dup	 1  14 'L__xy  xy!
  dup	 1  15 'L__xy  xy!
  dup	 3  16 'L__xy  xy!
  dup	 4  17 'L__xy  xy!
  dup	 5  18 'L__xy  xy!
  dup	 6  19 'L__xy  xy!
  dup	 7  20 'L__xy  xy!
  dup	 9  21 'L__xy  xy!
  dup   11  22 'L__xy  xy!
  dup   13  23 'L__xy  xy!
  dup   15  24 'L__xy  xy!
  dup   16  25 'L__xy  xy!
        17  26 'L__xy  xy!

  ( Compute the position of letter 'D' )
  'THRESHOLD'_#states alloc 'D__xy
  86
  dup  -22   0 'D__xy  xy!
  dup  -20   1 'D__xy  xy!
  dup  -18   2 'D__xy  xy!
  dup  -16   3 'D__xy  xy!
  dup  -14   4 'D__xy  xy!
  dup  -12   5 'D__xy  xy!
  dup  -10   6 'D__xy  xy!
  dup   -8   7 'D__xy  xy!
  dup   -6   8 'D__xy  xy!
  dup   -5   9 'D__xy  xy!
  dup   -3  10 'D__xy  xy!
  dup   -2  11 'D__xy  xy!
  dup   -1  12 'D__xy  xy!
  dup	 0  13 'D__xy  xy!
  dup	 1  14 'D__xy  xy!
  dup	 2  15 'D__xy  xy!
  dup	 3  16 'D__xy  xy!
  dup	 5  17 'D__xy  xy!
  dup	 6  18 'D__xy  xy!
  dup	 8  19 'D__xy  xy!
  dup   10  20 'D__xy  xy!
  dup   12  21 'D__xy  xy!
  dup   14  22 'D__xy  xy!
  dup   16  23 'D__xy  xy!
  dup   18  24 'D__xy  xy!
  dup   20  25 'D__xy  xy!
        22  26 'D__xy  xy!

  -1 Prelude_AnimTHR_I'0 !
  -1 Prelude_AnimTHR_I'1 !

  int nindex  nIndex 0!
  2 times
      I not if
	  0  'THRESHOLD'_#states 1-  1
      else
	  'THRESHOLD'_#states 2-  1  -1
      endif
      +make
	  I  nIndex @  prelude_AnimTHR_tbl  C!
	  nIndex 1+!
      endmake
  endtimes

  prelude_AnimTHR_index 0!

  ;  // Prelude_AnimTHR_init


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Prelude_AnimThreshold											        \\
// 	 Stack : ( --- )												        \\
// Description : While waiting user's action, screen animation.								        \\
//    Callers : Prelude												        \\
// 	 Calls : 2pages? ShowPage0 ShowPage1 SelectPage0 SelectPage1 draw_'THRESHOLD':5 'THRESHOLD'_#states:3		        \\
// 		 Prelude_AnimTHR_I'0:3 Prelude_AnimTHR_I'1:3 prelude_AnimTHR_tbl prelude_AnimTHR_index:5		        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: Prelude_AnimThreshold

  gmode_xor

  2pages? not if

      2 times

	  I not if
	      0	 'THRESHOLD'_#states 1-	 1
	  else
	      'THRESHOLD'_#states 2-  1	 -1
	  endif
	  +make

	      2 times
		  J draw_'THRESHOLD'
		  I not if  150 delay  endif
	      endtimes

	  endmake

      endtimes

  else

      prelude_AnimTHR_index @  prelude_AnimTHR_tbl  C@  const N
      prelude_AnimTHR_index @  2 mod  not if
	  SelectPage0
	  Prelude_AnimTHR_I'0 @ -1  !=  if
	      Prelude_AnimTHR_I'0 @ draw_'THRESHOLD'
	  endif
	  N draw_'THRESHOLD'
	  N Prelude_AnimTHR_I'0 !
	  ShowPage0
      else
	  SelectPage1
	  Prelude_AnimTHR_I'1 @ -1  !=  if
	      Prelude_AnimTHR_I'1 @ draw_'THRESHOLD'
	  endif
	  N draw_'THRESHOLD'
	  N Prelude_AnimTHR_I'1 !
	  ShowPage1
      endif

      prelude_AnimTHR_index 1+!
      prelude_AnimThr_index @  'THRESHOLD'_#states dup 2- +  ==	 if
	  prelude_AnimTHR_index 0!
      endif

  endif

  ;  // Prelude_AnimThreshold


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : GetGunAction												        \\
// 	 Stack : If an action key is hit: ( --- n tf )									        \\
// 		 If no action key is hit: ( --- ff )									        \\
// Description : Return the status of user's gun action.								        \\
//    Callers : Prelude												        \\
// 	 Calls : MOVE_GUN_LEFT MoveGunLeft_KeyIndice MOVE_GUN_RIGHT MoveGunRight_KeyIndice SHOOT_GUN ShootGun_KeyIndice	        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: GetGunAction

  MoveGunLeft_KeyIndice @ ?KeyState if
      MOVE_GUN_LEFT
      TRUE
  else
      MoveGunRight_KeyIndice @ ?KeyState if
	  MOVE_GUN_RIGHT
	  TRUE
      else
	  ShootGun_KeyIndice @ ?KeyState if
	      SHOOT_GUN
	      TRUE
	  else
	      FALSE
	  endif
      endif
  endif

  ;  // GetGunAction


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Prelude_gun												        \\
// 	 Stack : ( n --- )												        \\
// Description : This thread is responsable for moving the user gun on prelude.						        \\
//    Callers : Prelude												        \\
// 	 Calls : gun_X:11 Gun_Y:6 GUN_XINC:2 gun_xmin:2 gun_xmax:2 MOVE_GUN_LEFT MOVE_GUN_RIGHT SHOOT_GUN s_ArmedGun:6 2pages?  \\
// 		 ShowPage~ SelectPage~:2										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: Prelude_gun

  ( Compute new gun's position )
  case
      MOVE_GUN_LEFT  of	 gun_X @ GUN_XINC -  endof
      MOVE_GUN_RIGHT of	 gun_X @ GUN_XINC +  endof
      SHOOT_GUN	     of	 gun_X @	     endof
  endcase
  dup  gun_xmin @  <  if
       drop gun_xmin @
  else
      dup  gun_xmax @  >  if
	  drop gun_xmax @
      endif
  endif
  const x.new

  gun_X @  x.new  !=  if
      2pages? not if
	  gun_X @  gun_Y  s_ArmedGun  draw_sprite		   // Clear gun
	  x.new gun_X !
	  gun_X @  gun_Y  s_ArmedGun  draw_sprite		   // Draw gun at new place
      else
	  SelectPage~
	  gun_X @  gun_Y  s_ArmedGun  draw_sprite		   // Clear gun
	  x.new	   gun_Y  s_ArmedGun  draw_sprite		   // Draw gun at new place
	  ShowPage~
	  SelectPage~
	  gun_X @  gun_Y  s_ArmedGun  draw_sprite		   // Clear gun
	  x.new gun_X !
	  gun_X @  gun_Y  s_ArmedGun  draw_sprite		   // Draw gun at new place
      endif
  endif

  ;  // Prelude_gun


: zz

(  0 clear )
  InitSprites
  int x	 100 x !
  int y	 100 y !
  int old
  int new
  gmode_xor
  -1 old !

  int a
  int b
  int a'  a' 0!
  int b'  b' 0!

  begin
      2 times
	  I not if  0 6 1  else	 5 0 -1	 endif
	  +make

	      I case
		  0 of  s_alien_0  endof
		  1 of  s_alien_1  endof
		  2 of  s_alien_2  endof
		  3 of  s_alien_3  endof
		  4 of  s_alien_4  endof
		  5 of  s_alien_5  endof
		  6 of  s_alien_6  endof
	      endcase
	      new !

	      I case
		  0 of  J not if  0  0 else  1 -1 endif b ! a ! endof
		  1 of  J not if -1  1 else  0 -3 endif b ! a ! endof
		  2 of  J not if  0  3 else  0  0 endif b ! a ! endof
		  3 of  J not if  0  0 else  0  0 endif b ! a ! endof
		  4 of  J not if  0  0 else  0  0 endif b ! a ! endof
		  5 of  J not if  0  0 else -1  0 endif b ! a ! endof
		  6 of  J not if  1  0 else  0  0 endif b ! a ! endof
	      endcase

	      SelectPage~
	      old @ -1 != if
		  x  @ y    @   old @ draw_sprite	       // clear
	      endif
	      x @ a @ + y @ b @ + new @ draw_sprite	       // new
	      ShowPage~
	      SelectPage~
	      old @ -1 != if
		  x  @	 y  @   old @ draw_sprite		 // clear
	      endif
	      x @ a @ + y @ b @ + new @ draw_sprite	       // new
	      new @ old !

	      x @ a @ +	 x !
	      y @ b @ +	 y !

	      75 delay

	  endmake
      endtimes
  ?key until
  key drop
  ;


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Prelude												        \\
// 	 Stack : ( --- )												        \\
// Description : Wait a user's action on game's prelude.								        \\
//    Callers : threshold												        \\
// 	 Calls : 'T__xy 'H1_xy 'R__xy 'E__xy 'S__xy 'H2_xy 'O__xy 'L__xy 'D__xy Prelude_frame Prelude_AnimTHR_init	        \\
// 		 Prelude_AnimThreshold GetGunAction Prelude_gun								        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: Prelude

  Prelude_frame						       // Draw prelude drame
  Prelude_AnimTHR_init					       // Initializations needed for 'THRESHOLD' animation

  logical lStop	 lStop false!				       // Exit loop ?
  int ticks0  GetTicks ticks0 !
//  logical ?AnimThreshold
  begin
      GetGunAction if					       // ( --- n tf ) or ( --- ff )
	  Prelude_Gun					       // Gun movement
	  GetTicks 50 +	 ticks0	 !
//	    ?AnimThreshold true!
      else
//	  ?AnimThreshold false!
	  ?key if
	      key charuppercase case
		  'X of lStop true! endof		       // 'X -> eXit
'Y of zz endof
	      endcase
	  endif
      endif
      GetTicks ticks0 @ -  10  >=  ( ?AnimThreshold true?  or ) if
	  GetTicks ticks0 !
	  Prelude_AnimThreshold				       // Screen animation: 'THRESHOLD'
      endif
      lStop true?
  until

  free 'T__xy
  free 'H1_xy
  free 'R__xy
  free 'E__xy
  free 'S__xy
  free 'H2_xy
  free 'O__xy
  free 'L__xy
  free 'D__xy

  ;  // Prelude


: xx
  screen
  0 0 gotoxy 4 .R space key 27 == if quit endif
  graphics ;

// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Threshold												        \\
// 	 Stack : ( --- )												        \\
// Description : The top word of the application									        \\
//    Callers : g													        \\
// 	 Calls : ShowPage0 InitFramePosAndSize DrawFrame Prelude							        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: threshold
  screen ?UpdateCursor 1! graphics			       // Update cursor when drawing
  graphics 640x350x16					       // Select graphics mode 640x350x16
  InitFramePosAndSize					       // Init pos and size of all STATIC elements of the game (FRAME)
  DrawFrame						       // Install the frame of the game
  Prelude						       // Wait a user action on prelude game
//  FreeDynaVars						 // At the end of the application, free *all* dynamic variables.
  ShowPage0						       // Restore page 0
  ;  // threshold

: g
  threshold
  screen 22 20 gotoxy
  ;


( Print code/data size )
( -------------------- )

graphics
0 clear
infos

// " \\froth\\examples\\threshol.exe" save.exe cr
// " THRESHOL.EXE size is " type . "  bytes" type

