// ---------------------------------------------------------------------------------------------------------------------------- \\
//															        \\
// File: Trace.FTH      Author: Oliver SINGLA	 Date: Sept, 26, 94 Version: 1.00					        \\
// ----		        ------			 ----		    -------						        \\
//															        \\
// Description:	 Example on using trace manager (and also threads).							        \\
//															        \\
// Release:														        \\
//     2.00    Oct-02-95												        \\
//															        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\

( Force decimal for integer numeric I/O )
decimal

( Compiler strategy )
?speed true!						       // Choose speed vs. size
?safe true!						       // Run-time check stack under/overflow
?iarithoverflow true!					       // Run-time check integer arithmetic overflow
?chat true!						       // Print a dot for each compiled definition

( Empty word: this is the 1st word, we kill on each load )
forget <task>
: <task> ;

( Make SCREEN the context vocabulary )
screen

( Colors used )
black LightGray BackForeGround const C_NormalText	       // Normal text


// Constants positions
// -------------------

 0 const X_ActionsFrame					       ( # col )
 4 const Y_ActionsFrame					       ( # row )
27 const X_OutputFrame					       ( # col )
 4 const Y_OutputFrame					       ( # row )
 0 const X_InfosFrame					       // Informations panel: col
10 const Y_InfosFrame					       // Informations panel: row


// Nb of threads wanted
// --------------------

4 const THREADS_NB


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : t_int													        \\
// 	 Stack : Compile-time: ( --- )											        \\
// 		     Run-time: ( --- addr )										        \\
// Description : Create an integer variable, but one for each thread (very similar to INTEGER).				        \\
//    Callers : Counters												        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  t_int
   build>						       // Define COMPILE TIME sequence
       MaxNbThreads allot				       // ( --- addr )  Allocate space one cell for each thread
       drop						       // ( addr --- )  Throw away data address
   does>						       // Define RUN TIME seqeunce
       ActiveThread WordSize * +			       // Pointer on the cell specific to the active thread
   ;   // t_int


( Create one counter for each thread )
( ---------------------------------- )
t_int Counters


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawTittle												        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the tittle on screen										        \\
//    Callers : Frame													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: DrawTittle
  24 0 " Ŀ" !xystring
  24 1 "  Trace manager example - [OS] " !xystring
  24 2 " " !xystring
  ;  // DrawTittle


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawActionsFrame											        \\
// 	 Stack : ( --- )												        \\
// Description : Draw a box which contains instructions (user interface).						        \\
//    Callers : Frame													        \\
// 	 Calls : X_ActionsFrame Y_ActionsFrame										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: DrawActionsFrame
  X_ActionsFrame  Y_ActionsFrame
  ddup	  " Ŀ" !xystring
  1+ ddup "  +   Add a new msg       " !xystring
  1+ ddup "  -   Show trace messages " !xystring
  1+ ddup "  ESC Exit                " !xystring
  1+	  " " !xystring
  ;  // DrawActionsFrame


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawOutputFrame											        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the frame for threads output and informations.							        \\
//    Callers : Frame													        \\
// 	 Calls : X_OutputFrame Y_OutputFrame THREADS_NB									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: DrawOutputFrame
  X_OutputFrame Y_OutputFrame
  ddup	  " Ŀ" !xystring
  1  THREADS_NB	 do
      1+ ddup "  Thread #" !xystring
      2 I .R
      "  :                                      " !string
  enddo
  1+	  " " !xystring
  ;  // DrawOutputFrame


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DrawInfosFrame												        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the panel 'Nb of video page'.									        \\
//    Callers : Frame													        \\
// 	 Calls : X_InfosFrame:2 Y_InfosFrame:2										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: DrawInfosFrame
  X_InfosFrame  Y_InfosFrame				       // ( --- x y )
  ddup	  " Ŀ" !xystring
  1+ ddup "  Nb of video page :      " !xystring
  1+	  " " !xystring
  X_InfosFrame 21 +  Y_InfosFrame 1+  gotoXY
  #pages .						       // Print the nb of video pages available
  ;  // DrawInfosFrame


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : Frame													        \\
// 	 Stack : ( --- )												        \\
// Description : Draw the frame of the application.									        \\
//    Callers : TraceDemo												        \\
// 	 Calls : DrawTittle DrawActionsFrame DrawOutputFrame DrawInfosFrame						        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: Frame
  DrawTittle						       // Draw tittle application
  DrawActionsFrame					       // Draw actions frame
  DrawOutputFrame					       // Draw thread output thread
  DrawInfosFrame					       // Draw informations output
  ;  // Frame


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : ThreadOutputPosition											        \\
// 	 Stack : ( n --- x y )												        \\
// Description : Compute thread output position.									        \\
//    Callers : EachThread												        \\
// 	 Calls : X_OutputFrame Y_OutputFrame										        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: ThreadOutputPosition
  X_OutputFrame 14 +					       // ( --- x )
  Y_OutputFrame rot +					       // ( x --- x y )
  ;  // ThreadOutputPosition


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : EachThread												        \\
// 	 Stack : ( --- )												        \\
// Description : Thread word.												        \\
//    Callers :													        \\
// 	 Calls : Counters:3 ThreadOutputPosition									        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: EachThread
  Counters 0!						       // Initialize counter value
  ActiveThread ThreadOutputPosition			       // ( n --- n x y )  Give the output position
  begin							       // Forever loop
      ddup						       // ( x y --- x y x y )
      Counters @ <# #S #>  !xystring			       // Print value of counter
      Counters 1+!					       // Increments counter by one
      yield						       // Yield to another living thread
  again
  ;  // EachThread


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : TraceDemo												        \\
// 	 Stack : ( --- )												        \\
// Description : Top word of the application.										        \\
//    Callers :													        \\
// 	 Calls : C_NormalText X_ActionsFrame Y_ActionsFrame THREADS_NB:2 Counters Frame					        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
: TraceDemo
  C_NormalText ScreenBackground !			       // The screenbackground is Black
  ?UpdateCursor 1!					       // Update cursor when drawing
  cls							       // Clear screen

  Trace_Clear						       // Reset all previous msg into trace manager

  Frame							       // Draw the 'frame' of the application

  ( Run all the threads )
  1 THREADS_NB do
      0 NewThread EachThread				       // Create a new thread, no copy on private stack
      drop						       // ( n --- ) Drop the thread nb which is created
  enddo

  logical bStop	 bStop false!				       // Local flag: does stop application ?
  X_ActionsFrame 25 +  Y_ActionsFrame 3 +  gotoXY	       // Put cursor to a safe place
  ?UpdateCursor 0!					       // Do not update cursor position when drawing
  begin
      key						       // ( --- c ) Get user action
      case
	  0x1B	 of  bStop true!	 endof		       // ESC -> stop application
	  0x14A	 of  Trace_Page		 endof		       // Enter into the trace manager
	  0x14E	 of
		     Trace_Begin
		     " Thread #" Trace_Type
		     1 THREADS_NB rnd  dup Trace_.	       // Send msg to trace manager
		     "  : " Trace_Type
		     WordSize * Counters + @ Trace_.
		     Trace_End
		 endof
      endcase
      bStop true?
  until
  ;  // TraceDemo

: g TraceDemo ;


( Print code/data size )
( -------------------- )

infos

