// ---------------------------------------------------------------------------------------------------------------------------- \\
//															        \\
// File: GRCARD.FTH    Author: Oliver SINGLA    Date: Oct,11,95	   Version: 1.00					        \\
// ----		       ------		        ----		   -------						        \\
//															        \\
// Description:	   Look which VGA graphic card is used. When done, select a 43x132 text mode.				        \\
//															        \\
// Release:														        \\
//															        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\

( Force decimal for integer numeric I/O )
decimal

( Compiler strategy )
?speed true!						       // Choose speed vs. size
?safe true!						       // Run-time check stack under/overflow
?iarithoverflow true!					       // Run-time check integer arithmetic overflow
?chat false!						       // Print a dot for each compiled definition

( Empty word: this is the 1st word, we kill on each load )
forget <task>
:  <task> ;

( Make SCREEN the context vocabulary )
screen


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : DetectedCardIs												        \\
// 	 Stack : ( --- )												        \\
// Description : Just print the given msg.										        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  DetectedCardIs
   ." Detected VGA card: "
   ;   // DetectedCardIs


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : SelectedEdMode												        \\
// 	 Stack : ( addrz --- )												        \\
// Description : Show the description of the video mode used for editor session.					        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  SelectedEdMode
   cr
   ." Selected video mode for editor session: "
   type
   ;   // SelectedEdMode


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : !VideoMode												        \\
// 	 Stack : ( n --- )												        \\
// Description : Select the given video mode for the froth interpret.							        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  !VideoMode
   screen						       // Editor vocabulary
   SetVideoMode						       // The given screen text mode
   ;   // !VideoMode


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : !EditorVideoMode											        \\
// 	 Stack : ( n --- )												        \\
// Description : Select the given video mode for the editor session.							        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  !EditorVideoMode
   Editor						       // Editor vocabulary
   VideoMode !						       // The given screen text mode
   Screen						       // Screen vocabulary
   ;   // !EditorVideoMode


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : IsItParadise(WD)?											        \\
// 	 Stack : ( --- f )												        \\
// Description : Is-it a Paradise (Western Digital) VGA card ?								        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  isParadise(WD)?
   0xC007D [@] '=AGV == if
       0x03 !VideoMode					       // mode 3 is 25 x 80 x 16
//     0x54 !VideoMode					       // 43x132x16
       DetectedCardIs					       // Print a msg
       0x54 !EditorVideoMode				       // mode 0x54 is 43 x 132 x 16 (font 8 x 9)
       ." PARADISE (Western Digital)"
       " 0x54 (43 x 132 x 16, font 8 x 9)" SelectedEdMode
       TRUE						       // ( --- tf ) A Paradise card
   else
       FALSE						       // ( --- ff ) Not a Paradise card
   endif
   ;   // isParadise(WD)?


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : (isTrident)												        \\
// 	 Stack : ( --- f )												        \\
// Description : Is-it a TRIDENT VGA card ?										        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  (isTrident?)
   portio
   0x06 0x3CE cout
   0x3CF cin dup const old_misc_val
   // cr ." old_misc_val = " old_misc_val 02h.
   0x03 and 0x04 or const misc_val
   // cr ." misc_val = " misc_val 02h.
   misc_val 0x3CF cout
   0x0E 0x3C4 cout
   0x3C5 cin const old_val
   0x00 0x3C5 cout
   0x3C5 cin 0x0F and const val
   old_val 0x3C5 cout
   0x06 0x3CE cout
   old_misc_val 0x3CF cout
   // cr ." old_val = " old_val 02h.
   // cr ." val = " val 02h.
   val 0x02 >= if
       TRUE						       // ( --- tf ) A Paradise card
   else
       0x0F 0x3C4 cout
       0x00 0x3C5 cout
       0x3C5 cin
       FALSE						       // ( --- ff ) Not a Paradise card
   endif
drop false
   ;   // (isTrident?)


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : isTrident												        \\
// 	 Stack : ( --- f )												        \\
// Description : Is-it a TRIDENT VGA card ?										        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  isTrident?
   (isTrident?) dup if
       0x03 !VideoMode					       // mode 3 is 25 x 80 x 16
       DetectedCardIs					       // Print a msg
       0x55 !EditorVideoMode				       // mode 0x55
       ." TRIDENT"
       " 0x55 (43 x 132 x 16, font 8 x 11)" SelectedEdMode
   endif
   ;   // isTrident?


// ---------------------------------------------------------------------------------------------------------------------------- \\
// 	  Word : WhichVGACard?												        \\
// 	 Stack : ( --- )												        \\
// Description:	   Look which VGA graphic card is used. When done, select a 43x132 text mode.				        \\
//    Callers :													        \\
// 	 Calls :													        \\
// ---------------------------------------------------------------------------------------------------------------------------- \\
:  WhichVGACard?
   DetectedCardIs					       // Print a msg
   isParadise(WD)? not if
   isTrident? not if
       ." I don't know..."
   endif endif
   cr
   ;   // WhichVGACard?

